Package: genetics
Title: Population Genetics
Version: 1.3.8.1.3
Date: 2012-11-26
Author: Gregory Warnes, with contributions from Gregor Gorjanc,
        Friedrich Leisch, and Michael Man.
Maintainer: Gregory Warnes <greg@warnes.net>
Depends: combinat, gdata, gtools, MASS, mvtnorm
Description: Classes and methods for handling genetic data. Includes
        classes to represent genotypes and haplotypes at single markers
        up to multiple markers on multiple chromosomes. Function
        include allele frequencies, flagging homo/heterozygotes,
        flagging carriers of certain alleles, estimating and testing
        for Hardy-Weinberg disequilibrium, estimating and testing for
        linkage disequilibrium, ...
biocViews: Genetics
License: GPL
Packaged: 2021-03-01 06:56:06 UTC; hornik
Repository: CRAN
Date/Publication: 2021-03-01 07:10:07 UTC
NeedsCompilation: no
Built: R 4.6.0; ; 2026-04-25 18:04:39 UTC; unix

import(MASS, combinat)
export(
       HWE.chisq,
       HWE.exact,
       HWE.test,
       LD,
       #binsearch,  # moved to gtools
       ci.balance,
       diseq,
       diseq.ci,
       genotype,
       haplotype,
       is.genotype,
       is.haplotype,
       as.genotype,
       as.haplotype,
       allele.names,
       nallele,
       gregorius,
       homozygote,
       heterozygote,
       carrier,
       allele,
       allele.count,
       allele.names,
       #library.pos,
       locus,
       gene,
       marker,
       "locus<-",
       "gene<-",
       "marker<-",
       is.gene,
       is.locus,
       is.marker,
       getlocus,
       getmarker,
       getgene,
       makeGenotypes,
       makeHaplotypes,
       power.casectrl,
       LDtable,
       LDplot,
       as.factor,
       geno.as.array,
#       mknum,
       hap,
       hapshuffle,
       hapenum,
       hapfreq,
       hapmcmc,
       mourant,
       hapambig,
       write.pop.file,
       write.pedigree.file,
       write.marker.file,
       shortsummary.genotype,
       summary.LD.data.frame,
       as.genotype.allele.count,
       expectedGenotypes,
       expectedHaplotypes,
       order,
       genotypeOrder,
       "genotypeOrder<-",
       groupGenotype,
       "%in%"
       )

S3method(summary,genotype)
#S3method(allele.count.2,genotype)
#S3method(as.factor.allele,genotype)
S3method(as.factor,default)
S3method(as.factor,genotype)
#S3method('<-',locus)
#S3method('<-',marker)
#S3method('<-',gene)
S3method(print,gene)
S3method(print,locus)
S3method(print,marker)
S3method(as.character,locus)
S3method(as.character,gene)
S3method(as.character,marker)
S3method(carrier,genotype)
S3method(as.genotype,allele.count)
S3method(as.genotype,character)
S3method(as.genotype,default)
S3method(as.genotype,factor)
S3method(as.genotype,genotype)
S3method(as.genotype,haplotype)
S3method(as.genotype,table)
S3method(HWE.chisq,genotype)
S3method(HWE.test,genotype)
S3method(HWE.test,data.frame)
S3method(print,HWE.test)
S3method(LD,genotype)
S3method(LD,data.frame)
S3method(diseq,table)
S3method(diseq,genotype)
S3method(print,diseq)
S3method(print,genotype)
S3method('==',genotype)
S3method('==',haplotype)
S3method('%in%',default)
S3method('%in%',genotype)
S3method('%in%',haplotype)
S3method('[',genotype)
S3method('[',haplotype)
S3method('[<-',genotype)
S3method('[<-',haplotype)
S3method(heterozygote,genotype)
S3method(homozygote,genotype)
#S3method(print,allele.count)
S3method(print,allele.genotype)
#S3method(allele.count,genotype)
#S3method(allele,genotype)
S3method(print,LD)
S3method(print,LD.data.frame)
S3method(print,summary.LD.data.frame)
S3method(plot,LD.data.frame)
S3method(print,summary.genotype)
S3method(plot,genotype)
S3method(order, default)
S3method(order, genotype)
S3method(sort, genotype)

importFrom(stats, na.omit)
importFrom(gdata, interleave, trim, as.levelsMap, "mapLevels<-")
importFrom(mvtnorm, rmvnorm)
##importFrom(combinat, rmultz2) # combinat doesn't have a namesspace.
importFrom(gtools, binsearch, combinations, permutations)

importFrom("grDevices", "heat.colors")
importFrom("graphics", "abline", "axis", "barplot", "image", "legend",
           "locator", "matplot", "par", "plot", "strheight",
           "strwidth", "text", "title")
importFrom("stats", "anova", "chisq.test", "complete.cases", "lm",
           "optimize", "pchisq", "quantile", "runif")
importFrom("utils", "write.table")

S3method(summary, LD.data.frame)

Genetics version 0.03
=====================

The Genetics bundle consists of a collection of OO Perl modules designed to
facilitate the development of Perl scripts and applications in the area of
genetic analysis.  This includes modules encapsulating data related to genetic
analysis (Genetics::Object subclasses) and modules that implement an API for
interacting with these objects (Genetics::API and its derivatives), including
managing their persistence in a relational database (Genetics::API::DB::*).
The classes/objects, API and database are collectively referred to as GenPerl.
The original version of GenPerl was developed in the Research Department of
Genomica Corp. where it was used to produce research protytypes and to manage
and analyze data related to research projects with which Genomica was
involved.

See the files doc/index.html and doc/gptutorial.html for more information.

INSTALLATION

To install this module type the following:

   perl Makefile.PL
   make
   make test
   make install

DEPENDENCIES

This module requires these other modules and libraries:

  XML::Writer
  DBI
  DBD::mysql

The latter two are only needed if you want to use the GenPerl database
backend.  Obviously, you must also have a MySQL instance with an appropriate
database and schema in place.  If you have the necessary permissions,
something like the following should do the trick:

   % mysql 
   mysql> use mysql
   mysql> create database genperl;
   mysql> exit
   % mysql genperl < doc/createGenPerlSchema.sql

AUTHOR

Steve Mathias, smathias1@qwest.net

LICENCE

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

# narrow window, few few digits, quiet
options(width=50)
options(digits=2)
options(verbose=FALSE)


library(genetics)

# Load the data from a CSV file
data <- read.csv("example_data.csv")

# Convert genotype columns to genotype variables
data <- makeGenotypes(data)

## Annotate the genes 
marker(data$a1691g) <-
  marker(name="A1691G",
         type="SNP",
         locus.name="MBP2",
         chromosome=9, 
         arm="q", 
         index.start=35,
         bp.start=1691,
         relative.to="intron 1")


marker(data$c104t) <-
  marker(name="C-104T",
         type="SNP",
         locus.name="MBP2",
         chromosome=9, 
         arm="q", 
         index.start=35,
         bp.start=-104,
         relative.to="intron 1")

marker(data$c2249t) <-
  marker(name="C2249T",
         type="SNP",
         locus.name="MBP2",
         chromosome=9, 
         arm="q", 
         index.start=35,
         bp.start=2249,
         relative.to="intron 1")

# Look at some of the data
data[1:5,]

# Get allele information for c104t
summary(data$c104t)


# Check Hardy-Weinberg Equilibrium
HWE.test(data$c104t)

# Check Linkage Disequilibrium
ld <- LD(data)
ld # text display

pdf(file="LD.pdf")
LDtable(ld) # graphics display
dev.off()

summary(lm( DELTA.BMI ~ homozygote(c104t,'C') +
                        allele.count(a1691g, 'G') +
                        c2249t, data=data))

## Create a sample dataset with 3 SNP markers
set.seed(125141)

g1 <- sample( x=c('C/C', 'C/T', 'T/T'), 
              prob=c(.6,.2,.2), 100, replace=T)
g2 <- sample( x=c('A/A', 'A/G', 'G/G'), 
              prob=c(.6,.1,.5), 100, replace=T)
g3 <- sample( x=c('C/C', 'C/T', 'T/T'), 
              prob=c(.2,.4, 4), 100, replace=T)

y <- rnorm(100) + (g1=='C/C')  +
     0.25 * (g2=='A/A' | g2=='A/G')

pid <- formatC( abs(rnorm(100))*1e6, format="d", flag=0, width=8)

## Form into a data frame
data <- data.frame( PID=pid, DELTA.BMI=y, c104t=g1, a1691g=g2, c2249t=g3)

## Save as a file
write.table(data, file="example_data.csv", sep=",", quote=F, row.names=F)

# expectedGenotypes.R
#--------------------------------------------------------------------------
# What: Construct expected genotypes according to known allele variants
# Time-stamp: <2007-07-19 12:06:26 ggorjan>
#--------------------------------------------------------------------------

expectedGenotypes <- function(x, alleles=allele.names(x), ploidy=2,
                              sort=TRUE, haplotype=FALSE)
{
    ## Checks
    if (missing(x) && missing(alleles))
        stop("at least one of 'x' or 'alleles' must be given")
    if (!(missing(x) && !missing(alleles))) {
        if (!is.genotype(x)) stop("x must be of class 'genotype'")
    }

    nA <- length(alleles)
    if(nA==0)
      stop("Zero length alleles vector.")

    ## Add numbers to allele names, so we get result sorted by
    ## given allele names
    if (sort) {
        allelesOrig <- alleles
        alleles <- paste(1:nA, alleles, sep="")
    }

    ## Find possible genotypes according to allele variants
    if(!haplotype) {
      comb <- combinations(n=nA, r=ploidy, v=alleles, repeats.allowed=T)
    } else {
      comb <- permutations(n=nA, r=ploidy, v=alleles, repeats.allowed=T)
    }

    if (sort) {
        for (i in 1:nA) comb[comb == alleles[i]] <- allelesOrig[i]
    }

    ## Create a nice character vector of expected genotypes
    nC <- dim(comb)
    ret <- vector(mode="character", length=nC[1])
    ret <- paste(comb[, 1], comb[, 2], sep="/")
    if (ploidy > 2) {
        for (i in 3:nC[2]) {
            ret <- paste(ret, comb[, i], sep="/")
        }
    }
    return(ret)
}

expectedHaplotypes <- function(x, alleles=allele.names(x), ploidy=2,
                              sort=TRUE, haplotype=TRUE)
{
  expectedGenotypes(x=x, alleles=alleles, ploidy=ploidy,
                    sort=sort, haplotype=haplotype)
}

#--------------------------------------------------------------------------
# expectedGenotypes.R ends here

# $Id: genotype.R 1337 2008-04-30 00:54:56Z warnes $

genotype  <- function(a1, a2=NULL, alleles=NULL, sep="/",
                      remove.spaces=TRUE,
                      reorder=c("yes", "no", "default", "ascii", "freq"),
                      allow.partial.missing=FALSE,
                      locus=NULL, genotypeOrder=NULL)
{
    if(missing(reorder))
      reorder  <- "freq"
    else
      reorder <- match.arg(reorder)

    if(is.genotype(a1)){
        a1  <-  as.character(a1)
        ## ignore a2
        a2 <- NULL
    }
    else
      {
        a1.d <- dim(a1)
        a1 <- as.character(a1)
        dim(a1) <- a1.d
        a1[is.na(a1)] <- ""   # necessary because of bug in grep & friends,
                              # will be fixed in 1.7.1
      }

    if(!is.null(a2))
      {
        a2.d <- dim(a2)
        a2 <- as.character(a2)
        dim(a2) <- a2.d
        a1[is.na(a1)] <- ""  # necessary because of bugs in grep & friends
                             # will be fixed in 1.7.1
      }

    if(remove.spaces)
    {
        a1dim <- dim(a1)
        a1  <-  gsub("[ \t]", "", a1)
        dim(a1) <- a1dim
        if(!is.null(a2))
            a2  <-  gsub("[ \t]", "", a2)
    }

    if(!is.null(dim(a1)) && ncol(a1) > 1)
        parts <- a1[,1:2]
    else if(!is.null(a2))
        parts  <- cbind(a1,a2)
    else
      {
        # if sep is empty, assume allele names are single characters
        # pasted together
        if(sep=="")
          sep  <- 1

        # Based on the value of sep, reformat into our standard
        # name-slash-name format
        if (is.character(sep) )
          {
            part.list   <- strsplit(a1,sep)
            part.list[ sapply(part.list, length)==0] <- NA

            ## Handle missing / empty values correctly. 
            ## Without this, empty elements are silently dropped
            ## and/or cause errors

            # only first field was given
            half.empties  <- lapply(part.list, length)==1
            part.list[half.empties]  <-  lapply(part.list[half.empties],c,NA)

            # neither field was given
            empties  <- is.na(a1) | lapply(part.list, length)==0
            part.list[empties]  <- list(c(NA,NA))

            parts <- matrix(unlist(part.list),ncol=2,byrow=TRUE)

          }
        else if (is.numeric(sep))
          parts  <- cbind( substring(a1,1,sep), substring(a1,sep+1,9999))
        else
          stop(paste("I don't know how to handle sep=",sep))
      }

    mode(parts) <- "character"  # needed for bare NA's o

    # convert entirely whitespace alleles to NAs
    temp  <- grep("^[ \t]*$", parts)
    parts[temp]  <-  NA

    #parts[parts=="NA"]  <-  NA

    if(missing(alleles) || is.null(alleles))
      alleles <- unique(c(na.omit(parts)))
    else
      {
        which.alleles  <- !(parts %in% alleles)
        ## Skipping NA's
        which.alleles <- which.alleles & !is.na(parts)
        if(any(which.alleles))
          {
            warning("Found data values not matching specified alleles. ",
                    "Converting to NA.")
            parts[which.alleles] <- NA
          }
      }

    if(!allow.partial.missing)
      parts[is.na(parts[,1]) | is.na(parts[,2]),]  <- c(NA,NA)

    if(reorder!="no")
    {
        if(reorder=="ascii")
        {
            alleles <-  sort(alleles)
        }
        else if(reorder=="freq")
        {
            ## get reordering of alleles by frequency
            tmp  <- names(rev(sort(table(parts))))
            alleles  <- unique(c(tmp,alleles))
        }

        reorder  <- function( x, alleles)
        {
            tmp <- match( x, alleles )
            x[order(tmp)]
        }

        parts  <- t(apply(parts,1, reorder, alleles))

      }

    tmp  <-  ifelse( is.na(parts[,1]) & is.na(parts[,2]),
                    NA,
                    apply(parts,1,paste,collapse="/") )

    object  <- factor( tmp )

    # force "NA" not to be a factor level
    ll  <- levels(object)  <-  na.omit(levels(object))

    class(object)  <-  c("genotype","factor")
    attr(object,"allele.names")  <- alleles
    attr(object,"allele.map")  <- do.call("rbind", strsplit(ll, "/"))

    genotypeOrder(object) <- genotypeOrder

    if(is.null(locus) || is.locus(locus)  )
      attr(object,"locus")  <- locus
    else
      stop("parameter locus must be of class locus")
    return(object)
  }

is.genotype  <- function(x)
    inherits(x, "genotype")

is.haplotype  <- function(x)
    inherits(x, "haplotype")

###
### Haplotype -- differs only in that order of a1,a2 is considered siginificant
###
haplotype <- function (a1, a2 = NULL, alleles = NULL, sep = "/",
                       remove.spaces = TRUE, reorder = "no",
                       allow.partial.missing = FALSE, locus = NULL,
                       genotypeOrder=NULL) 
{
    retval <- genotype(a1 = a1, a2 = a2, alleles = alleles, sep = sep, 
                       remove.spaces = remove.spaces, reorder = reorder,
                       allow.partial.missing = allow.partial.missing, 
                       locus = locus, genotypeOrder=genotypeOrder)
    class(retval) <- c("haplotype", "genotype", "factor")
    retval
}


as.haplotype  <- function(x,...)
{
    retval <- as.genotype(x,...,reorder="no")
    class(retval)  <- c("haplotype","genotype","factor")
    retval
}

###
### Display by giving values plus list of alleles
###

print.genotype  <-  function(x,...)
  {
    if(!is.null(attr(x,"locus")))
        print(attr(x,"locus"))
    print(as.character(x))
    cat("Alleles:", allele.names(x), "\n" )
    invisible(x)
  }

###
### Conversion Functions
###

as.genotype  <- function (x,...) 
  UseMethod("as.genotype")

# Do we want to do this?
as.genotype.default  <-  function(x,...)
  genotype(x,...)

#  stop("No method to convert this object to a genotype")

# for characters, and factors, just do the standard thing (factors get
# implicitly converted to characters so both have the same effect.
as.genotype.character  <-  function(x,...)
  genotype(x,...)

as.genotype.factor  <-  function(x,...)
  genotype(as.character(x),...)

as.genotype.genotype  <- function(x,...)
  return(x)

as.genotype.haplotype  <- function(x,...)
  return(x)


## genotype.allele.counts give the count of each allele type as a
## matrix.  Collapse back into the form we need

as.genotype.allele.count  <- function(x, alleles=c("A","B"), ...)
  {
    if(!is.matrix(x) & !is.data.frame(x) )
      {
        x  <- cbind(x, 2-x)
        colnames(x)  <- alleles
      }

    if(any(x > 2, na.rm=TRUE) || any( x < 0, na.rm=TRUE ) )
      stop("Allele counts must be in {0,1,2}")

    allele.names  <-  colnames(x)
    tmp  <-  apply(x, 1, function(y)
                    rep( colnames(x), ifelse(is.na(y), 0, y) ))

    if(!is.matrix(tmp))
      retval  <-  genotype(sapply(tmp,paste,collapse="/"), alleles=alleles, ...)
    else
      retval  <- genotype(a1=tmp[1,], a2=tmp[2,], ... )
    return(retval)
  }

allele.count.2.genotype  <-  function(...)
  as.genotype.allele.count(...)



as.genotype.table <- function(x, alleles, ...)
  {
    #if(missing(alleles)) alleles <- unique(unlist(dimnames(x)))
    tmp <- outer( rownames(x), colnames(x), paste, sep="/")
    retval <- genotype( rep(tmp,x), alleles=alleles )
    retval
  }


###
### Equality test for genotype, assumes allele order is _not_ significant
###
"==.genotype"  <-  function(x,y)
  {
    if(!is.genotype(y))
      y <- as.genotype(y)

    x.a1  <- allele(x,1)
    x.a2  <- allele(x,2)

    y.a1  <- allele(y,1)
    y.a2  <- allele(y,2)

    return( (x.a1==y.a1 & x.a2==y.a2) | (x.a1==y.a2 & x.a2==y.a1) )
  }

###
### Equality test for haplotype, assumes allele order _is_ significant
###
"==.haplotype"  <-  function(x,y)
  {
    if(!is.genotype(y))
      y <- as.haplotype(y)

    x.a1  <- allele(x,1)
    x.a2  <- allele(x,2)

    y.a1  <- allele(y,1)
    y.a2  <- allele(y,2)

    return( x.a1==y.a1 & x.a2==y.a2 )
  }

###
### is.element i.e. %in%
###

"%in%" <- function(x, table)
  UseMethod("%in%")

## Get default method for %in% from base package
"%in%.default" <- get("%in%", pos="package:base")

"%in%.genotype" <- function(x, table)
{
  xA1  <- allele(x, 1)
  xA2  <- allele(x, 2)

  x1 <- paste(xA1, xA2, sep="/")
  x2 <- paste(xA2, xA1, sep="/")

  ## Return
  ((x1 %in% table) | (x2 %in% table))
}

"%in%.haplotype" <- function(x, table)
  as.character(x) %in% as.character(table)

###
### Extract the first and/or second allele.
###
### By default, return a 2 column matrix containing both alleles
###

#allele  <- function (x,...) 
#  UseMethod("allele")


#allele.genotype  <-  function(x, which=c(1,2) )
allele  <-  function(x, which=c(1,2) )
  {
    alleles.x  <- attr(x,"allele.map")
    retval  <- alleles.x[as.integer(x),which]
    attr(retval,"locus")  <- attr(x,"locus")
    attr(retval,"which")  <- which
    attr(retval,"allele.names")  <- allele.names(x)    
    #class(retval)  <- c("allele.genotype", class(retval))
    return( retval)
  }

as.factor  <- function(x, ...)
  UseMethod("as.factor")

as.factor.default  <- get("as.factor",pos="package:base")
formals(as.factor.default) <- c(formals(as.factor.default),alist(...= ))

as.factor.genotype <- function(x, ...)
  {
    attr(x,"class") <- "factor"
    attr(x,"allele.names") <- NULL
    attr(x,"allele.map") <- NULL
    attr(x,"locus") <- NULL
    attr(x,"genotypeOrder") <- NULL
    x
  }

as.factor.allele.genotype  <-  function(x,...)
  factor(x,levels=allele.names(x))

print.allele.genotype  <- function(x,...)
  {
    if(!is.null(attr(x,"locus")))
      print(attr(x,"locus"))
    cat("Allele(s):", attr(x,"which"), "\n")
    attr(x, "which")  <-  attr(x, "class") <- attr(x,"locus") <- attr(x,"allele.names")  <- NULL
    NextMethod("print",x)
  }


###
### Obtain the count of the number of copies of alleles for each individual
###
### By default, return a matrix containing the counts for all possible allele values.
###

#allele.count  <- function (x,...) 
#  UseMethod("allele.count")

#allele.count.default <- function (x, ... )
#  {
#    x <- as.genotype(x)
#    allele.count(x, ...)
#  }

#allele.count.genotype  <- function(x, allele.name=allele.names(x),

allele.count  <- function(x, allele.name=allele.names(x),
                          any=!missing(allele.name), na.rm=FALSE)
{
  if(!missing(allele.name) && length(allele.name)==1)
    {
      a.1  <- allele(x,1)
      a.2  <- allele(x,2)

      retval  <- ifelse(is.na(a.1) | is.na(a.2),
                        ifelse(na.rm, 0, NA),
                        (a.1==allele.name) + (a.2==allele.name) )
#      class(retval)  <- "allele.count"
      attr(retval,"allele") <- allele.name
      attr(retval,"locus")  <- attr(x,"locus")
      return(retval)
    }
  else
    {
      retval  <- sapply( allele.name, function(y) allele.count(x,y))
      if(any==TRUE && is.matrix(retval)  )
      retval  <- apply(retval,1,sum,na.rm=na.rm)
      if(na.rm) retval[is.na(retval)]  <- 0
#      class(retval)  <- "allele.count"
      attr(retval,"locus")  <- attr(x,"locus")
      return(retval)
    }

}


#print.allele.count  <- function(x,...)
#  { 
#    if(!is.null(attr(x,"locus")))
#        print(attr(x,"locus"))
#    
#    if(is.null(attr(x,"allele")))
#      cat("Allele Counts:\n")
#    else
#      cat("Allele Count (", attr(x,"allele"), " allele):\n", sep="")
#    val  <- x
#    attr(val,"class")  <- NULL
#    attr(val,"allele")  <- NULL
#    print(val)
#    invisible(x)
#  }

###
### Check for the presence of alleles for each individual
###
### By default, return a matrix containing indicators for all possible
### allele values except the last.
###
#
#allele.ind  <-  function(x,allele)
#  {
##    if(missing(allele))
##      stop("Alleles to test must be specified")
##    if(length(allele)==1)
#      retval  <- allele.count(x,allele) > 0
##    else
##      retval  <- apply(allele.count(x,allele) ,1,sum) > 0
#
#      if(missing(allele))
#          allele  <-  colnames(retval)
#      attr(retval,"allele")  <- allele
#      attr(retval,"locus")  <- attr(x,"locus")
#      class(retval)  <-  "allele.ind"
#      return(retval)
#  }

#print.allele.ind  <- function(x,...)
#  {
#    if(!is.null(attr(x,"locus")))
#      print(attr(x,"locus"))
#    
#    cat("Indicator(s) for allele(s):", attr(x,"allele"), "\n")
#    attr(x,"locus")  <-  attr(x,"class")  <- attr(x,"allele")  <-  NULL
#    NextMethod("print",x)
#  }

###
### Methods for creating subsets based on a genotype
###

homozygote  <- function (x,allele.name,...) 
  UseMethod("homozygote")

homozygote.genotype  <-  function(x,allele.name,...)
  {
    a1  <- allele(x,1)
    a2  <- allele(x,2)
    if(missing(allele.name))
      retval  <- ifelse( is.na(a1) | is.na(a2), NA, a1==a2 )
    else
      retval  <- ifelse( is.na(a1) | is.na(a2), NA,
                         a1==allele.name & a2==allele.name )
    attr(retval,"locus")  <-  attr(x,"locus")
#    class(retval)  <-  "homozygote"
    return(retval)
  }

#print.homozygote  <- function(x,...)
#  {
#    if(!is.null(attr(x,"locus")))
#      print(attr(x,"locus"))
#    
#    cat("Homozygote Indicators:\n")
#    attr(x,"locus")  <-  attr(x,"class")  <- attr(x,"allele")  <-  NULL
#    NextMethod("print",x)
#  }


heterozygote  <- function (x,allele.name,...) 
  UseMethod("heterozygote")

heterozygote.genotype  <-  function(x,allele.name,...)
  {
  {
    a1  <- allele(x,1)
    a2  <- allele(x,2)
    if(missing(allele.name))
      retval  <- ifelse( is.na(a1) | is.na(a2), NA, !a1==a2 )
    else
      retval  <- ((a1 %in% allele.name) | (a2 %in% allele.name)) &
                 (a1 != a2)
    attr(retval,"locus")  <-  attr(x,"locus")
#    class(retval)  <-  "homozygote"
    return(retval)
  }
  }

#print.heterozygote  <- function(x,...)
#  {
#    if(!is.null(attr(x,"locus")))
#      print(attr(x,"locus"))
#    
#    cat("Heterozygote Indicators:\n")
#    attr(x,"locus")  <-  attr(x,"class")  <- attr(x,"allele")  <-  NULL
#    NextMethod("print",x)
#  }

carrier <- function (x,allele.name,...) 
  UseMethod("carrier")

carrier.genotype  <-  function(x, allele.name=allele.names(x),
                                   any=!missing(allele.name), na.rm=FALSE, ...)
{
  retval  <- allele.count(x,allele.name=allele.name,any=any,na.rm=na.rm) > 0

  attr(retval,"allele")  <- retval["allele"]
  attr(retval,"locus")  <-  attr(x,"locus")
#  class(retval)  <- "carrier"
  return(retval)
}


#print.carrier  <- function(x,...)
#  {
#    if(!is.null(attr(x,"locus")))
#      print(attr(x,"locus"))
#    
#    cat("Carrier Indicator(s) for allele(s):", attr(x,"allele"), "\n")
#    attr(x,"locus")  <-  attr(x,"class")  <- attr(x,"allele")  <-  NULL
#    NextMethod("print",unclass(x))
#  }


###
###
###

allele.names<- function(x)
  {
    retval  <- attr(x,"allele.names")
    if(is.null(retval))
      retval  <- x$allele.names
    return(retval)
  }

###
### Subset method
###

"[.genotype"  <-  function(x, i, drop=FALSE)
  {
    allelesOld <- attr(x, "allele.names")
    retval  <- NextMethod("[")

    # force "NA" not to be a factor level
    ll  <- levels(retval)  <-  na.omit(levels(retval))

    class(retval)  <-  c("genotype","factor")

    if(drop) {
      alleles <- unique( unlist(strsplit(ll, "/") ) )
    } else {
      alleles <- attr(x, "allele.names")
    }
    attr(retval,"allele.names")  <- alleles
    attr(retval,"allele.map")  <- do.call("rbind", strsplit(ll, "/"))
    attr(retval,"locus")  <- attr(x,"locus")
    attr(retval,"label")  <-  attr(x,"label")
    goCur <- attr(x, "genotypeOrder")
    if(drop) {
      ## Removing genotype names having dropped alleles
      allelesOld <- allelesOld[!(allelesOld %in% alleles)]
      tmp <- allele(as.haplotype(goCur))
      test <- tmp %in% allelesOld
      test <- rowSums(matrix(test, ncol=2)) > 0
      attr(retval, "genotypeOrder") <- goCur[!test]
    } else {
      attr(retval, "genotypeOrder") <- goCur
    }
    return(retval)
  }

"[.haplotype"  <-  function(x, i, drop=FALSE)
  {
    retval  <- NextMethod("[")
    class(retval) <- c("haplotype","genotype","factor")
    retval
  }

###
### Subset Assigment method
###

"[<-.genotype"  <-  function(x, i, value)
  {
    ## Special case for insertion of NA and "" values
    if(all( is.na(value) | as.character(value)<="" ) )
      {
        x.class <- class(x)
        x <- unclass(x)
        x[i] <- NA
        class(x) <- x.class
        return(x)
      }
      
    if(!is.genotype(value))
      {
        value <- genotype(value)
      }

    lx <- levels(x)
    lv <- levels(value)
    ax <- allele.names(x)
    av <- allele.names(value)

    m  <- is.na(match(av,ax) )
    if( any( m  )  )
       warning(paste("Adding new allele name:", av[m], "\n"))

    la <- unique(c(lx,lv))
    aa <- unique(c(ax,av))

    cx <- class(x)
    nas <- is.na(x)

    data  <-  match(levels(value)[value],la)

    class(x) <- NULL
    x[i] <- data
    attr(x, "levels") <- la
    map  <- attr(x, "allele.map")  <- do.call("rbind", strsplit(la, "/"))
    attr(x, "allele.names")  <- aa
    goCur <- attr(x, "genotypeOrder")
    goAll <- expectedGenotypes(alleles=aa, haplotype=TRUE)
    attr(x, "genotypeOrder") <- c(goCur, goAll[!(goAll %in% goCur)])
    class(x) <- cx
    x
  }

"[<-.haplotype"  <-  function(x, i, value)
  {
    if(!is.haplotype(value))
      stop("Assigned value must be of class haplotype.")
    NextMethod("[<-")
  }

nallele <- function(x)
  length(allele.names(x))

genotypeOrder <- function(x)
  attr(x, "genotypeOrder")

"genotypeOrder<-" <- function(x, value)
{
  if(!is.genotype(x)) stop("'x' must be of a genotype class")
  alleles <- allele.names(x)

  goAll <- expectedGenotypes(alleles=alleles, haplotype=TRUE)
  goDef <- unique(sort(as.character(x)))

  if(is.null(value)) {
    attr(x, "genotypeOrder") <- goAll
  } else {
    value <- unique(value)

    ## Stop msg says all
    parts <- strsplit(x=value, split="/")
    parts <- sapply(parts, c)
    test <- !(parts %in% alleles)
    if(any(test))
      stop("adding genotype names with alleles that are not in the data")

    ## Any genotypes in the data that are not in value?
    test <- !(goDef %in% value)
    if(any(test)) {

      ## These values are in all possible genotypes/haplotypes
      testDefinAll <- goDef[test] %in% goAll
      ## but not in value
      testDefinAllnotVa <- !(goDef[testDefinAll] %in% value)
      goPos <- goDef[testDefinAllnotVa]

      ## We could simply add goPos to value now. However, A/B in goPos
      ## should also match B/A, since genotype() allows reordering of
      ## original data and additionally we want this to work also for
      ## haplotype.

      ## Extend value first. We do not do this before, since one
      ## might not necessarily like to have B/A together with A/B in
      ## first place.
      value <- .genotype2Haplotype(x=value)
      ## Remove heterozygos matches in goPos
      test <- !(goPos %in% value)
      goPos <- goPos[test]

      ## Add goPos to the end of value
      if(any(test)) value <- c(value, goPos)

      ## If there are still some values in all, but not in value
      ## now, we just add them at the end
      testGOnotAll <- !(goAll %in% value)
      if(any(testGOnotAll)) value <- c(value, goAll[testGOnotAll])
    } else {
      value <- .genotype2Haplotype(x=value)
    }
    attr(x, "genotypeOrder") <- value
  }
  x
}

.genotype2Haplotype <- function(x)
{
  ## Internal function
  ##
  ## Returns a character vector of possible haplotypes for given genotypes
  ## in such a way that for say c("A/A", "A/B", "B/B") you get c("A/A",
  ## "A/B", "B/A", "B/B") i.e. "B/A" comes directly after "A/B"!
  ##
  ## x - character, vector of genotype values in form allele1/allele2
  ##
  ## Details
  ## Unique values of x are taken i.e. first occurrence prevails
  ##
  ## Example
  ## .genotype2Haplotype(c("A/A", "A/B", "B/B"))
  ## "A/A" "A/B" "B/A" "B/B"
  ## .genotype2Haplotype(c("B/B", "A/B", "A/A"))
  ## "B/B" "A/B" "B/A" "A/A"

  x <- unique(x)
  N <- length(x)
  parts <- .genotype2Allele(x=x)
  parts <- rbind(parts, parts[, 2:1])
  ind <- rep(1:N, each=2) + c(0, N)
  parts <- parts[ind, ]
  parts <- unique(paste(parts[, 1], parts[, 2], sep="/"))
  parts
}

.genotype2Allele <- function(x)
{
  ## Internal function
  ##
  ## Returns a matrix of alleles from a character vector of genotype names
  ##
  ## x - character, vector of genotype values in form allele1/allele2
  ##
  ## Details:
  ## Coercing to character is done for x.
  ##
  ## Example
  ## .genotype2Allele(c("A/A", "A/B", "B/B"))
  ##      [,1] [,2]
  ## [1,] "A"  "A"
  ## [2,] "A"  "B"
  ## [3,] "B"  "B"

  parts <- strsplit(x=as.character(x), split="/")
  parts <- t(sapply(parts, c))
  parts
}

# $Id: plot.LD.R 150 2003-06-04 21:22:57Z warnesgr $

plot.LD.data.frame <- function(x,
                               digits=3,

                               colorcut=c(0,0.01, 0.025, 0.5, 0.1, 1),
                               colors=heat.colors(length(colorcut)),
                               textcol="black",

                               marker,
                               which="D'",
                               distance,
                               ...)
  {
    oldpar <- par("mfrow")
    
    par(mfrow=c(1,2))

    LDtable(x, digits=digits, colorcut=colorcut, colors=colors,
            textcol=textcol, ...)
    LDplot(x, marker=marker, which=which, distance=distance, ...)
    
    par(mfrow=oldpar)
    invisible()
  }


LDtable <- function(x, 
                    colorcut=c(0,0.01, 0.025, 0.5, 0.1, 1),
                    colors=heat.colors(length(colorcut)),
                    textcol="black",
                    digits=3,
                    show.all=FALSE,
                    which=c("D", "D'", "r", "X^2", "P-value", "n"),
                    colorize="P-value",
                    cex,
                    ...)
  {
    if(! colorize %in% names(x))
      stop(colorize, " not an element of ", deparse(substitute(x)) )

    datatab <- summary(x)
    
    missmatch <- which[!(which %in% names(x))]
    if(length(missmatch)>0)
      stop(missmatch, " not an element of ", deparse(substitute(x)) )

    matform <- function( value, template )
      {
        dim(value) <- dim(template)
        dimnames(value) <- dimnames(template)
        value
      }
    
    tmp <- cut(x[[colorize]], colorcut, include.lowest=TRUE)
    colormat <- matform(as.numeric(tmp), x[[colorize]] )
    n <- matform( paste("(",x$n,")",sep="" ), x$n)

    if(!show.all)
      { # remove blank row/column
        colormat <- colormat[-nrow(colormat),-1, drop=FALSE]
        n <- n[-nrow(n),-1, drop=FALSE]
      }

    #
    # color coded frame boxes
    #
    image(x=1:ncol(colormat), y=1:ncol(colormat),
          z=t(colormat[nrow(colormat):1,]),
          col=colors, xlab="Marker 2\n\n", ylab="Marker 1",
          xaxt="n", yaxt="n",...)
    
    abline(v=-0.5 + 1:(ncol(colormat)+1))
    abline(h=-0.5 + 1:(nrow(colormat)+1))
    
    axis(3, 1:ncol(colormat), colnames(colormat) )
    axis(2, 1:nrow(colormat), rev(rownames(colormat)) )
    
    #
    # text in boxes
    #
    cex.old <- par("cex")

    if(missing(cex))
      cex <-min( c(1/10, 1/(length(which)+1 ) ) /
                 c(strwidth("W"), strheight("W")*1.5))
    
    par(cex=cex)

    lineheight <- strheight("W")*1.5
    center <- lineheight * length(which)/2
    
    for(i in 1:length(which))
      {
        displaymat <- x[[which[i]]]

        if(!show.all)
          displaymat <- displaymat[-nrow(displaymat),-1, drop=FALSE]
        
        if( which[i]=="P-value" )
          displaymat <- format.pval(displaymat, digits=digits)
        else if (which[i]!="n")
          displaymat <- format(displaymat, digits=digits)

        displaymat[] <- gsub("NA.*", "", as.character(displaymat))
        
        text(x=col(colormat),
             y=nrow(colormat) - row(colormat)+ 1 + center - lineheight*(i-1),
             displaymat,
             col=textcol,
             adj=c(0.5, 1)
             )
      }

    text(x=1, y=1, paste(which, collapse="\n"), adj=c(0.5,0.5) )

    par(cex=cex.old)
    
    #
    # title
    #
    title(main="Linkage Disequilibrium\n")

    invisible(colormat)
  }



LDplot <- function(x, 
                   digits=3,
                   marker,
                   distance,
                   which=c("D", "D'", "r", "X^2", "P-value", "n", " "),
                   ...)
{
  which = match.arg(which)
  
  if(missing(marker))
    marker <- colnames(x[[which]])
  else if (is.numeric(marker))
    marker <- colnames(x[[which]])[marker]
  
  datamat <- ifelse( is.na(x[[which]]), t(x[[which]]), x[[which]])

  if(which %in% c("D'","r") )
    diag(datamat) <- 1.0
  else if (which=="P-value")
    diag(datamat) <- 0.0
  
  dimnames(datamat) <- dimnames(x[[which]])
  
  if(missing(distance)) distance <- 1:ncol(datamat)
  distance <- matrix(distance, ncol=ncol(datamat), nrow=nrow(datamat),
                     byrow=TRUE)
  dimnames(distance) <- dimnames(datamat)
  
  matplot(x=t(distance[marker,,drop=FALSE]),
          t(datamat[marker,,drop=FALSE]),
          type="b", 
          xlab="Marker",
          ylab=paste("Linkage Disequilibrium: ", which, sep=""),
          xaxt="n",
          ... )
  
  axis(1, distance[1,], paste(1:ncol(datamat), colnames(datamat), sep=": " ))
  
  title("Pairwise Disequilibrium Plot")

  invisible()
}

# power calculation for case-control design (default: case:control = 1:1)
# Author: Michael Man
#   Date: May 5, 2004
#      N: total number of subjects
#  gamma: relative risk in multiplicative model;
#         not used in Dominant or Recessive model (assume A as protective allele)
#      p: frequency of A allele
#     kp: prevalence of disease
#  alpha: significance level
#     fc: fraction of cases
#     pi: probability of 'aa' genotype has the disease
#   minh: mode of inheritance
# reference: Long, A. D. and C. H. Langley (1997). Genetic analysis of complex traits. Science 275: 1328.
#            Agresti, A. (2002) Categorical Data Analysis. Second Edition, p243.
# ( modified from pbsize{gap} )
# requirement: It is recommended to use R 1.9.0 or above.  The function 'qchisq' in earlier version
#              has problem with large noncentrality parameter.


# under HWE                       AA       Aa       aa       
#   fHW = p(genotype)        = c( p^2,     2pq,     q^2 )     

# model specification 
#   f.mod = relative risk    = c(gamma^2,  gamma,    1 )    # multiplicative model
#   f.mod =                  = c(  0,        0,      1 )    #       dominant model
#   f.mod =                  = c(  0,        1,      1 )    #      recessive model

# conditional prob.
#   p(D|genotype) = f.mod*pi = c(gamma^2,  gamma,    1 )*pi

# population joint prob. (f.mod = 1 under Ho)
#   Case     p(D,     genotype) = p(genotype)*     p(D|genotype)  = fHW*   f.mod*pi
#   Control  p(D_not, genotype) = p(genotype)*(1 - p(D|genotype)) = fHW*(1-f.mod*pi)

# population conditional prob. (f.mod = 1 under Ho)
#   Case     p(genotype|D)     = p(D    , genotype)/P(D    ) = P(D    , genotype)/sum(P(D    , genotype)) = fHW*   f.mod*pi  /    sum(fHW*f.mod*pi)
#   Control  p(genotype|D_not) = p(D_not, genotype)/P(D_not) = P(D_not, genotype)/sum(P(D_not, genotype)) = fHW*(1-f.mod*pi) / (1-sum(fHW*f.mod*pi))

# sample or allocation probability
#   1:1 case-control design  p(D|Sample) = fc = 1/2
#   1:2 case-control design                fc = 1/3
#   a prospective design                   fc = sum(fHW*f.mod*pi)

# sample joint prob. (f.mod = 1 under Ho)
# for prospective design, this is the same as population joint prob. since 'fc' cancels out with 'sum(fHW*f.mod*pi)' 
#   Case     p(genotype,D    |sample) = p(genotype|D    )*     p(D|Sample)  =    fc *fHW*   f.mod*pi  /    sum(fHW*f.mod*pi)
#   Control  p(genotype,D_not|sample) = p(genotype|D_not)*(1 - p(D|Sample)) = (1-fc)*fHW*(1-f.mod*pi) / (1-sum(fHW*f.mod*pi))


## power.casectrl <- function (N, gamma = 4.5, p = 0.15, kp=.1, alpha=.05, fc=0.5,
##                              minh=c('multiplicative', 'dominant','recessive','partialrecessive')) 
## {
##   minh <- match.arg(minh)
##   if ( !all(gamma > 0, N > 0) ) stop('N and gamma must be greater than 0')
##   if ( min(p, kp, alpha, fc) <= 0 | max(p, kp, alpha, fc) >=1 ) stop('p, kp, alpha, and fc must be between 0 and 1.') 
##   f.mod <- switch(minh,
##          multiplicative = c(gamma^2, gamma, 1),
##       partialrecessive = c(gamma,       1, 1),           
##          dominant       = c(      0,     0, 1),
##          recessive      = c(      0,     1, 1)  ) 
##   q <- 1 - p
##   fhw <- c(p^2, 2*p*q, q^2)
##   pi <- kp/sum(f.mod*fhw)
##   if (pi <= 0 | pi >=1) {
##     warning('The combination of p, kp, and gamma produces an unrealistic value of pi.')
##     ret <- NA
##   } else {
##     fe  <- rbind(fhw, fhw)
##     dimnames(fe) <- list(c("Case", "Control"), c("AA", "Aa", "aa"))
##     f <- fe*rbind(f.mod*pi, 1-f.mod*pi)
##     Pct <- apply(f, 1, sum)
##     f2 <-  f *c(fc, 1-fc)/Pct   # normalize the frequencies for each row
##     fe2 <- fe*c(fc, 1-fc)  
##     fe2; apply(fe2, 1, sum); f2; apply(f2, 1, sum)  
##     lambda <- sum((f2-fe2)^2/fe2)*N
##     ret <- 1 - pchisq(qchisq(1-alpha, df=1), df=1, ncp=lambda, lower.tail=T)
##   }
##   ret
## }

power.casectrl <- function (...)
{
  .Deprecated("'GPC', 'GeneticPower.Quantitative.Factor', or 'GeneticPower.Quantitative.Numeric in the BioConductor GeneticsDesign package")
}

## $Id: zzz.R 1330 2007-11-20 20:23:12Z warnes $

# Obsoleted by Proper use the DESCRIPTION and NAMESPACE files
.onAttach <- function(libname, pkgname)
{
	packageStartupMessage("\n")
        packageStartupMessage("NOTE: THIS PACKAGE IS NOW OBSOLETE.\n")
	packageStartupMessage("\n")
	packageStartupMessage("  The R-Genetics project has developed an set of enhanced genetics\n")
	packageStartupMessage("  packages to replace 'genetics'. Please visit the project homepage\n")
        packageStartupMessage("  at http://rgenetics.org for informtion.\n")
	packageStartupMessage("\n")
	
}

# $Id: write.pedigree.file.R 666 2006-03-10 18:19:34Z nj7w $

write.pedigree.file <- function(data,
                                family, pid, father, mother, sex,
                                file="pedigree.txt"
                                )
  {
    # pedigree file format
    # --------------------
    #
    # <family> <pid> <father> <mother> <sex> <genotype_1> ... <genotype_n>
    #
    # <family> is a unique identifier for each family, and within each family
    # <pid> is a unique identifier for an individual.
    # <father> and <mother> identify the individuals father and mother
    #    (if this line refers to a founder, these should be set to
    #     zero).
    #  <sex> denotes the individuals sex, using the convention
    #     1=male, 2=female.
    #
    # Each <genotype> is encoded as two integer allele numbers.

    if(missing(family)) 
      family <- 1:nrow(data)
    if(missing(pid))
      pid <- 1:nrow(data)
    if(missing(father))
      father <- rep(0,nrow(data))
    if(missing(mother))
      mother <- rep(0,nrow(data))
    if(missing(sex))
      sex <- rep(0,nrow(data))

    pedigree <- list()
    pedigree$family <- format(family)
    pedigree$pid    <- format(pid)
    pedigree$father <- format(father)
    pedigree$mother <- format(mother)
    pedigree$sex    <- format(sex)

    which <- sapply(data, is.genotype)
    if(!all(which)) warning("Data contianed non-genotype variables.",
                            " These have been ignored: ",
                            paste(colnames(data)[!which]) )

    data <- data[,which]

    allele.number <- function(g, ind) {
      as.numeric(factor(allele(g, ind),
                        levels = allele.names(g)))
    }

    for( col in names(data) )
      {
        name.1 <- paste(col,".1")
        name.2 <- paste(col,".2")

        

       ## allele.number <- function(g, ind)
       ##   as.numeric(as.factor(allele(g,ind), levels=allele.names(g, ind) ))
        
        pedigree[[name.1]] <- allele.number( data[[col]], 1)
        pedigree[[name.2]] <- allele.number( data[[col]], 2)
      }
    pedigree <- as.data.frame(pedigree)
    
    # NA's are represented as 0
    pedigree[is.na(pedigree)] <- 0

    write.table(pedigree,   file=file, sep=" ", row.names=FALSE,
                col.names=F, quote=F)
  }

write.marker.file<-function(data, location, file="marker.txt")
{
    # marker.map file format
    # --------------------
    #
    # MARKERID   NAME      LOCATION
    # <col#>     <name>    <location>

    which <- sapply(data, is.genotype)
    if(!all(which)) warning("Data contianed non-genotype variables.",
                            " These have been ignored: ",
                            paste(colnames(data)[!which]) )

    data <- data[,which]
    if(missing(location)) location <- 1:ncol(data)

    ## Create marker.map file data frame
    marker.map <- cbind(
                             formatC(1:ncol(data), width=8),
                             formatC(colnames(data), width=8, flag="-"),
                             formatC(location, width=8)
                             )

    marker.map <- rbind(c("MARKERID", "NAME    ","LOCATION"), marker.map)
    
    write.table(marker.map, file=file, sep="   ", row.names=FALSE,
                col.names=F, quote=F)    
}

# $Id: HWE.exact.R 114 2003-05-22 17:25:23Z warnesgr $
#
# Based on code submitted by David Duffy <davidD@qimr.edu.au>
#
# Exact test for HWE: 2 alleles
#
# See eg Emigh TH.  Comparison of tests for Hardy-Weinberg Equilibrium.
#  Biometrics 1980; 36: 627-642
#

HWE.exact <- function(x)
{
  if(!is.genotype(x))
    stop("x must be of class 'genotype' or 'haplotype'")

  nallele <- length(na.omit(allele.names(x)))

  if(nallele != 2)
    stop("Exact HWE test can only be computed for 2 markers with 2 alleles")


  allele.tab <- table( factor(allele(x,1), levels=allele.names(x)),
                       factor(allele(x,2), levels=allele.names(x)) )

  n11 <- allele.tab[1,1]
  n12 <- allele.tab[1,2] + allele.tab[2,1]
  n22 <- allele.tab[2,2]
  
  n1 <- 2*n11+n12
  n2 <- 2*n22+n12

  
  dhwe2 <- function(n11, n12, n22) {
    f <- function(x) lgamma(x+1)
    n <- n11+n12+n22
    n1 <- 2*n11+n12
    n2 <- 2*n22+n12
    exp(log(2)*(n12) + f(n) - f(n11) - f(n12) - f(n22) - f(2*n) +
        f(n1) + f(n2))
  }


  x12 <- seq(n1 %% 2,min(n1,n2),2)
  x11 <- (n1-x12)/2
  x22 <- (n2-x12)/2
  dist <- data.frame(n11=x11,n12=x12,n22=x22,density=dhwe2(x11,x12,x22))
  dist <- dist[order(dist$density),]

  STATISTIC <- c("N11"=n11,"N12"=n12,"N22"=n22)
  PARAMETER <- c("N1"=n1,"N2"=n2)
  PVAL <- cumsum(dist$density)[dist$n11==n11 &
                               dist$n12==n12 &
                               dist$n22==n22] 
  METHOD <- "Exact Test for Hardy-Weinberg Equilibrium"
  DNAME <- deparse(substitute(x))
  
  retval <- list(statistic = STATISTIC, parameter = PARAMETER, 
        p.value = PVAL, method = METHOD, data.name = DNAME, observed = x)
  class(retval) = "htest"
  return(retval)

}

# $Id: print.LD.R 395 2005-10-04 23:43:31Z warnes $

print.LD <- function(x, digits=getOption("digits"), ...)
  {
    saveopt <- options("digits")
    options(digits=digits)
    cat("\n")
    cat("Pairwise LD\n")
    cat("-----------\n")

    est <- t(as.matrix( c(D=x$"D","D'"=x$"D'","Corr"=x$"r")))
    rownames(est) <- "Estimates:"
    print(est)
    cat("\n")

    test <- t(as.matrix( c("X^2"=x$"X^2", "P-value"=x$"P-value",
                           "N"=x$"n") ) )
    rownames(test) <- "LD Test:"
    print(test)
    cat("\n")

    options(saveopt)
    invisible(x)
  }


summary.LD.data.frame <- function(object, digits=getOption("digits"),
                                which=c("D", "D'", "r", "X^2",
                                        "P-value", "n", " "),
                                rowsep, show.all=FALSE,
                                ...)
  {

    if(missing(rowsep))
      if(length(which)==1)
        rowsep <- NULL
      else
        rowsep <- " "

    if(is.null(rowsep))
      blank <- NULL
    else
      blank <- matrix(rowsep, ncol=ncol(object$"D"), nrow=nrow(object$"D"))
    


    saveopt <- options("digits")
    options(digits=digits)

    
    pdat <- list()
    for(name in which)
         pdat[[name]] <- object[[name]]
    
    tab <- interleave(
                      "D" = if('D' %in% names(pdat)) pdat$D else NULL,
                      "D'" = pdat$"D'",
                      "Corr." = pdat$"r",
                      "X^2"= pdat$"X^2",
                      "P-value" = pdat$"P-value",
                      "n" = pdat$"n",
                      " "=blank,
                      sep=" "
                      )

    statlist <- which[ ! (which %in% c("P-value", "n", " ") ) ]
    statlist[statlist=="X^2"] <- "X\\^2"

    formatlist <- sapply( statlist, function(object) grep(object, rownames(tab) ) )
    formatlist <- unique(sort(unlist(formatlist)))
    
    pvallist   <- grep( "P-value", rownames(tab) )
    
    tab[formatlist,] <- formatC(as.numeric(tab[formatlist,]), digits=digits,
                                format="f")
    tab[pvallist,] <- apply(object$"P-value", c(1,2),
                            function(object)trim(format.pval(object, digits=digits)))
    
    tab[trim(tab)=="NA"] <- NA

    if(!show.all)
      {
         # drop blank row/column
        entrylen <- nrow(tab)/nrow(object$n)
        tab <- tab[1:(nrow(tab) - entrylen),-1]
      }

    
    options(saveopt)
    class(tab) <- "summary.LD.data.frame"
    tab
  }

print.summary.LD.data.frame <- function(x, digits=getOption("digits"), ...)
{
  cat("\n")
  cat("Pairwise LD\n")
  cat("-----------\n")

  print(as.matrix(unclass(x)), digits=digits, quote=FALSE, 
        na.print="    ", right=TRUE) 
        
  cat("\n")

  invisible(x)

  
}


print.LD.data.frame <- function(x, ...)
  print(summary(x))

# $Id: hapmcmc.R 1352 2012-08-14 14:21:35Z warnes $
#
# Code contributed by David Duffy <davidD@qumr.edu.au>:
#
# "If you are interested, this is a toy/prototype for haplotyping via MCMC.  
#  It is much slower than Dan Schaid's haplo.em, but does give the same
#  answers ;)"

#
# Routines for handling genotypes
#
# Convert "1/2" to 1,2
#
geno.as.array <- function(genotypes,renumber=FALSE,miss=NULL,gtp.sep="/") {
  mknum<-function(genotypes, renumber=FALSE, gtp.sep="/") {
    alleles<- strsplit(genotypes, gtp.sep)
    gtp<-cbind(sapply(alleles, function(x) x[1], simplify=TRUE),
               sapply(alleles, function(x) x[2], simplify=TRUE))
    if (renumber) {
      alleles<-unique(unlist(alleles))
      gtp[,1]<-as.numeric(factor(gtp[,1],levels=alleles))
      gtp[,2]<-as.numeric(factor(gtp[,2],levels=alleles))
    }
    if (is.null(miss)) {
      gtp[is.na(genotypes),]<-NA
    }else{
      gtp[is.na(genotypes),]<-miss
    } 
    gtp
  } 
  if (is.null(ncol(genotypes)) || ncol(genotypes)==1) {
    res<-mknum(genotypes, renumber=renumber)
  }else{
    res<-data.frame(mknum(genotypes[,1], renumber=renumber))
    for(i in 2:ncol(genotypes)) {
      res<-cbind(res,mknum(genotypes[,i], renumber=renumber))
    } 
    colnames(res)<-c(t(outer(names(genotypes),1:2,paste,sep="."))) 
  } 
  apply(res,2,as.character)
} 
#
hap <- function(genotypes) {
  res<-geno.as.array(genotypes)
  nc<-ncol(res)
  hap1<-res[,seq(1,nc,2)]
  hap2<-res[,seq(2,nc,2)]
  loci<-colnames(genotypes)
  colnames(hap2)<-colnames(hap1)<-loci
  list(hap1=hap1, hap2=hap2, class="haplotype")
}
hapshuffle <- function(haplotypes, hfreq=NULL, ambiguous=NULL, verbose=FALSE, set) {
  if (is.null(hfreq)) hfreq<-hapfreq(haplotypes, set=set)  
  if (is.null(ambiguous)) ambiguous<-hapambig(haplotypes)
  nloci<-ncol(haplotypes$hap1)
  nobs<-nrow(haplotypes$hap1)
  for(ind in ambiguous) {
    prop<-curr<-list(hap1=haplotypes$hap1[ind,], hap2=haplotypes$hap2[ind,])
    swap<-sample(c(TRUE,FALSE),nloci,replace=TRUE)
    if (any(swap)) {
      tmp<-prop$hap1[swap]
      prop$hap1[swap]<-prop$hap2[swap]
      prop$hap2[swap]<-tmp
    }
    o1<-paste(curr$hap1,collapse=":")  
    o2<-paste(curr$hap2,collapse=":")  
    n1<-paste(prop$hap1,collapse=":")  
    n2<-paste(prop$hap2,collapse=":")  
    pos.o1<-match(o1,names(hfreq))
    pos.o2<-match(o2,names(hfreq))
    pos.n1<-match(n1,names(hfreq))
    pos.n2<-match(n2,names(hfreq))
    pn<-(hfreq[pos.n1]+0.5)*(hfreq[pos.n2]+0.5)
    po<-(hfreq[pos.o1]+0.5)*(hfreq[pos.o2]+0.5)
    qa<-pn/po
    if (verbose) cat("Person ",ind," ",qa," ",o1,"/",o2," -> ",n1,"/",n2,sep="")
    if (qa>runif(1)) {
      if (verbose) cat(" Accepted\n")
      haplotypes$hap1[ind,]<-prop$hap1
      haplotypes$hap2[ind,]<-prop$hap2
      hfreq[pos.n1]<-hfreq[pos.n1]+1
      hfreq[pos.n2]<-hfreq[pos.n2]+1
      hfreq[pos.o1]<-hfreq[pos.o1]-1
      hfreq[pos.o2]<-hfreq[pos.o2]-1
    }else if (verbose) {
      cat(" Unchanged\n")
    }
  }
  list(hfreq=hfreq, haplotypes=haplotypes, class="hapshuffle")
}

hapambig <- function(haplotypes) {
  which(apply(haplotypes$hap1!=haplotypes$hap2,1,sum)>1)
}

hapenum <- function(haplotypes) {
  dat<-rbind(haplotypes$hap1, haplotypes$hap2)
  dat<-dat[complete.cases(dat),]
  set<-unique(dat[,1])
  for(i in 2:ncol(dat)) set<-outer(set,unique(dat[,i]),paste,sep=":") 
  factor(set)
}

hapfreq <- function(haplotypes, set=NULL) {
  if (is.null(set)) set<-hapenum(haplotypes)
  hap1<-apply(haplotypes$hap1[complete.cases(haplotypes$hap1),],1,paste,collapse=":")
  hap2<-apply(haplotypes$hap2[complete.cases(haplotypes$hap2),],1,paste,collapse=":")
  dat<-c(hap1,hap2)
  table(factor(dat,levels=set))
}

hapmcmc <- function(gtp, B=1000) {
  tot<-2*nrow(gtp)
  hap.dat<-hap(gtp)
  hap.set<-hapenum(hap.dat)
  hap.amb<-hapambig(hap.dat)
  hap.new<-list(hfreq=hapfreq(hap.dat, set=hap.set), haplotypes=hap.dat)
  res<-matrix(nrow=B, ncol=length(hap.set))
  colnames(res)<-as.character(hap.set)
  rownames(res)<-1:B
  for(i in 1:B) {
    hap.new<-hapshuffle(hap.new$haplotypes,hfreq=hap.new$hfreq,ambiguous=hap.amb, set=hap.new$set)
    res[i,]<-hap.new$hfreq
  }
  apply(res,2,mean)/tot
}

mourant <- function(n) {
  tab<-matrix(c(91,32,5,147,78,17,85,75,7), nrow=3)
  rownames(tab)<-c("M/M","M/N","N/N")
  colnames(tab)<-c("S/S","S/s","s/s")
  dat<-as.data.frame.table(tab)
  p<-dat$Freq/sum(dat$Freq)
  dat[sample(1:nrow(dat),n,replace=TRUE,prob=p),1:2]
}

library(genetics)

suppressWarnings(RNGversion("3.5.0"))

set.seed(12345)

# Create a test data set where there are several genotypes in columns
# of the form "A/T".
test1 <- data.frame(Tmt=sample(c("Control","Trt1","Trt2"),20, replace=TRUE),
                G1=sample(c("A/T","T/T","T/A",NA),20, replace=TRUE),
                N1=rnorm(20),
                I1=sample(1:100,20,replace=TRUE),
                G2=paste(sample(c("134","138","140","142","146"),20,
                                replace=TRUE),
                         sample(c("134","138","140","142","146"),20,
                                replace=TRUE),
                         sep=" / "),
                G3=sample(c("A /T","T /T","T /A"),20, replace=TRUE),
                comment=sample(c("Possible Bad Data/Lab Error",""),20,
                               rep=TRUE)
                )
test1


# now automatically convert genotype columns
geno1 <- makeGenotypes(test1)
geno1



set.seed(12345)

# Create a test data set where there are several genotypes in columns
# of the form "A_T".
test1.b <- data.frame(Tmt=sample(c("Control","Trt1","Trt2"),20, replace=TRUE),
                G1=sample(c("A_T","T_T","T_A",NA),20, replace=TRUE),
                N1=rnorm(20),
                I1=sample(1:100,20,replace=TRUE),
                G2=paste(sample(c("134","138","140","142","146"),20,
                                replace=TRUE),
                         sample(c("134","138","140","142","146"),20,
                                replace=TRUE),
                         sep=" _ "),
                G3=sample(c("A _T","T _T","T _A"),20, replace=TRUE),
                comment=sample(c("Possible Bad Data/Lab Error",""),20,
                               rep=TRUE)
                )
# now automatically convert genotype columns
geno1.b <- makeGenotypes(test1.b, sep="_")

stopifnot(identical(geno1,geno1.b))

power.casectrl.plot <- function (N, gamma=1.6, p=1:9/10, kp=0.1, alpha=0.05, fc=0.5,
                                 minh=c('multiplicative', 'dominant','recessive'),
                                 Nsnp=1, vary=c('prevalence','SNPs'), ylim=c(0,1), PLOT=T, ... )
{
  minh <- match.arg(minh)
  vary <- match.arg(vary)
  if (length(p)<2) stop('Must have more than 1 value in p.')
  if (length(kp) > 1 & length(Nsnp) > 1) stop("Nsnps and kp can't be all > 1.")
  if (vary=='prevalence') {
    cmd <- expression(tapply(p, p, function(x, ...) power.casectrl(p=x,...), N=N, gamma=gamma, kp=kp[j], alpha=alpha/Nsnp, fc=fc, minh=minh))
    Xvary <- kp 
  } else if (vary=='SNPs') {
    cmd <- expression(tapply(p, p, function(x, ...) power.casectrl(p=x,...), N=N, gamma=gamma, kp=kp, alpha=alpha/Nsnp[j], fc=fc, minh=minh))
    Xvary <- Nsnp 
  }
  J <- length(Xvary)
  ret <- matrix(NA, ncol=J, nrow=length(p))
  colnames(ret) <- paste(vary, '=', Xvary)
  for (j in 1:J) ret[,j] <- eval(cmd)

  if (PLOT) {
    nc <- 1:ncol(ret)
    subt <- paste("( RR", gamma, "; total subjects", N,"; SNPs", Nsnp[1], "; prevalence", kp[1],
                   "; mode of inheritance:", minh, "; overall sig.level", alpha, ")" )
    matplot(p, ret, type="l", ylim=ylim, lty=1, col=nc, xlab="Allele Frequency", ylab="Power", sub=subt, ...)
    abline(h=c(.8), lty=1)
    legend( locator(1), colnames(ret), lty=1, col=nc )
  }
  ret
}


### simple simulation for two group design
pw <- function(n1, n2=n1*(1-fc)/fc, fc=.5, pi=0, me1=50, me2=45, sd1=10, sd2=10, TEST=F){
  covm <- matrix(c(1,    pi,    pi,    1   ), nrow=2)*
          matrix(c(sd1^2, sd1*sd2, sd1*sd2, sd2^2), nrow=2)
  x1 <- rmvnorm(n=n1, mean=c(me1,me1), sigma=covm)
  x2 <- rmvnorm(n=n2, mean=c(me1,me2), sigma=covm)
  x <- data.frame(rbind(x1,x2), Trt=c(rep(0,n1), rep(1,n2)))
  colnames(x) <- c('X', 'Y', 'Trt')
  mod <- lm(Y~X+Trt, data=x)
  if (TEST) {
    print( summary(mod) )
    plot(Y~X+Trt, data=x)
  }
  ret <- anova(mod, test='F')['Trt','Pr(>F)']
  ret
}

# power calculation for studies using baseline measure 
#   - simulation: continuous response, baseline and genotype as covariate (ANCOVA)
#   - can specify various modes of inheritance ('additive', 'dominant','recessive')
#   - use compound symmetry for covariance matrix 
# Author: Michael Man
#   Date: June 22, 2004
#      N: total number of subjects
#      p: frequency of A allele
#  alpha: significance level                     (used only in 'power.genotype.conti')
#    Rep: number of iterations to generate power (used only in 'power.genotype.conti')
#     pi: correlation coefficient
#    me1: mean of control group
#  delta: treatment/genotype effect
#  sd1/2: standard deviation of the control and treatment groups
#   minh: mode of inheritance
# genotype.delta: the effect due to individual genotype effect or overall effect
# Factor: whether treat 'Trt' as a factor in 'x'
#   TEST: debug
# reference:
#   Frison and Pocock (1992) "Repeated measures in clinical trials: analysis using mean summary statistics
#     and its implications for design" Statistics in Medicine 11:1685-1704
#   Vickers (2001) "The use of percentage change from baseline as an outcome in a controlled trial is
#     statistically inefficient: a simulation study" BMC Med Res Methodol. 2001; 1 (1): 6
# requirement: need library 'mvtnorm'

simu.genotype.conti <- function (N, p=0.15, pi=0, me1=50, me2=me1, delta=-5, sd1=10, sd2=10, TEST=F,
                                minh=c('additive', 'dominant','recessive'), genotype.delta=T, Factor=F) 
{
  minh <- match.arg(minh)
  if ( min(N, sd1, sd2)<0 ) stop('N, sd1, and sd2 must be greater than 0')
  if ( p<=0 | abs(pi)<0 | p>=1 | abs(pi)>1 ) stop('p and abs(pi) must be between 0 and 1.') 
  f.mod <- switch(minh,
         dominant       = c(      0,     1, 1),
         additive       = c(      0 ,  0.5, 1),
         recessive      = c(      0,     0, 1)  ) 
  q <- 1 - p
  fhw <- c(q^2, 2*p*q, p^2)  # major allele first
  nhw <- round(N*fhw)
  if (sum(nhw)!=N) nhw[3] <- N-sum(nhw[1:2])
  covm <- matrix(c(1,    pi,    pi,    1   ), nrow=2)*
          matrix(c(sd1^2, sd1*sd2, sd1*sd2, sd2^2), nrow=2)
  if (!genotype.delta) delta <- delta/sum(fhw*f.mod)  # convert to overall delta due to all genotypes

  ## Give explicit variable definitions for variables created by the
  ## next loop to avoid unassigned variable message. 
  x1 <- x2 <- x3 <- t1 <- t2 <- t3 <- NULL 

  for (i in 1:3) {
    if (nhw[i]!=0) {
      assign(paste('x',i,sep=''), rmvnorm(n=nhw[i], mean=c(me1,me2+f.mod[i]*delta), sigma=covm))
      assign(paste('t',i,sep=''), rep(f.mod[i],nhw[i]))
    } else {
      assign(paste('x',i,sep=''), NULL)
      assign(paste('t',i,sep=''), NULL)
    }
  }
  x <- data.frame(rbind(x1,x2,x3), Trt=c(t1,t2,t3))
  if (Factor) x$Trt <- as.factor(x$Trt)
  colnames(x) <- c('X', 'Y', 'Trt')
  mod <- lm(Y~X+Trt, data=x)   # ANCOVA
  if (TEST) {
    print( summary(mod) )
    plot(Y~X+Trt, data=x)
  }
  ret <- anova(mod, test='F')['Trt','Pr(>F)']
  ret
}

### power calculation
power.genotype.conti <- function(N, Rep=100, alpha=.05, ...){
  pval <- sapply(rep(N, Rep), FUN=simu.genotype.conti, ...)
  power <- length(pval[pval<=alpha])/length(pval)
  power
}

# $Id: ci.balance.R 453 2005-11-09 17:04:02Z warnes $

ci.balance <- function(x, est, confidence=0.95, alpha=1-confidence,
                       minval, maxval, na.rm=TRUE)
  {
    if( any(is.na(x) ) )
      {
        if( na.rm)
          x <- na.omit(x)
        else
          stop("Missing values and NaN's not allowed if `na.rm' is FALSE.")
      }

    if(missing(minval))
      {
        minval <- min(x)
        minname <- "min(x)"
      }
    else
      minname <- "Lower Boundary"
    
    if(missing(maxval))
      {
        maxval <- max(x)
        maxname <- "max(x)"
      }
    else
      maxname <- "Upper Boundary"
    
    x <- sort(x)
    n <- length(x)
    half.window <- n * (1-alpha) / 2
    n.below <- sum( x < est ) + sum( x==est )/2
    n.above <- sum( x > est ) + sum( x==est )/2 

    overflow.upper <- max(0, half.window - n.above )
    overflow.lower <- max(0, half.window - n.below ) 
    
    lower.n <- max(1, floor  ( n.below - half.window - overflow.upper ) )
    upper.n <- min(n, ceiling( n - (n.above - half.window - overflow.lower ) ) )
    
    ci <- c( x[lower.n], x[upper.n] )
    names(ci) <- paste( format( c(lower.n, upper.n)/n*100,digits=3 ), "%", sep="")

    if(overflow.lower>0)
      {
        lower.n <- minname
        names(ci)[1] <- minname
        ci[1] <- minval
      }
    if(overflow.upper>0)
      {
        upper.n <- maxname
        names(ci)[2] <- maxname
        ci[2] <- maxval
      }
    

    return(
           list(
                ci=ci,
                overflow.upper=overflow.upper,
                overflow.lower=overflow.lower,
                n.above=n.above,
                n.below=n.below,
                lower.n=lower.n,
                upper.n=upper.n
                )
           )
  }

# $Id: locus.R 1337 2008-04-30 00:54:56Z warnes $

getlocus  <- function(x,...)
{
  if(is.locus(x))
    return(x)
  else if(!is.null(x["locus"]))
        return(x["locus"])
  else if(!is.null(attr(x,"locus")))
       return(attr(x,"locus"))
  else
    NULL
}

getmarker <- getgene <- getlocus

locus  <- function(name, chromosome, arm=c("p","q","long","short",NA),
                   index.start=NULL, index.end=NULL)
  {
    
    object  <-  list()

    if(!missing(name))
      object$name  <- name
    
    if(!missing(chromosome))
      object$chromosome <- chromosome
    
    if(!missing(arm))
      {
        arm  <- match.arg( arm )
        object$arm  <- switch( arm, p="p", q="q", long="p", short="q")
      }
    if(!missing(index.start))
      object$index.start  <- index.start
    if(!missing(index.end))
      object$index.end  <- index.end
    
    class(object)  <- "locus"
    return(object)
  }


gene  <-  function(name, chromosome, arm=c("p","q","long","short"),
                   index.start, index.end=NULL)
{
  object  <- locus(name, chromosome, arm, index.start, index.end)
  class(object)  <- c("gene","locus")
  object
}


marker <- function(name, type,
                   locus.name, bp.start, bp.end=NULL, relative.to=NULL,
                   ...
                   )
{
  if(is.locus(locus.name))
      object <- locus.name
  else
    object  <-  locus(locus.name, ...)

  if(!missing(name))
    object$marker.name  <- name

  if(!missing(type))
    object$type  <- type

  if(!missing(bp.start))
    object$bp.start  <- bp.start

  if(!missing(bp.end))
    object$bp.end  <- bp.end

  if(!missing(relative.to))
    object$relative.to  <- relative.to
  
  class(object)  <- c("marker","locus")
  object
}

is.locus  <- function(x)
    inherits(x, "locus")

is.gene  <- function(x)
    inherits(x, "gene")

is.marker  <- function(x)
    inherits(x, "marker")



as.character.locus  <- function(x,...)
  {
    loc <- paste( x$chromosome, x$arm, x$index.start, sep="" )
    if( !is.null(x$index.end ) && x$index.start != x$index.end )
      loc  <- paste(loc, "-", x$index.end, sep="")
    loc
  }

as.character.gene  <- function(x,...)
  as.character.locus(x,...)

as.character.marker  <- function(x,...)
  {
    loc  <- as.character.locus(x)
    loc  <- paste(loc, ":", x$bp.start, sep="")
    if(!is.null(x$bp.end)) loc  <-  paste(loc, "-", x$bp.end, sep="")
    loc
  }

print.locus  <-  function(x,...)
  {
    cat("Locus: ", x$name, " (", as.character.locus(x), ")\n", sep="" )
  }

print.gene  <-  function(x,...)
  {
    cat("Gene: ", x$name, " (", as.character.locus(x), ")\n", sep="" )
  }

print.marker  <- function(x,...)
  {
    cat("Marker: ", paste(x$name,":",x$marker.name,sep=""),
        " (", as.character.marker(x), ")\tType: ",x$type,"\n", sep="" )
  }


"locus<-" <- function(x,value)
  {
    attr(x,"locus") <- value
    x
  }


"marker<-" <- "gene<-" <-  get("locus<-")

# $Id: HWE.test.R 1225 2007-05-29 19:29:10Z warnes $

### Hardy-Weinberg Equilibrium Disequlibrium Estimates, Confidence
### Intervals, and P-values
###



HWE.test <- function(x, ...)
{
	UseMethod("HWE.test")
}


HWE.test.genotype <- function(x, exact=nallele(x)==2,
                              simulate.p.value=!exact, B=10000,
                              conf=0.95, ci.B=1000, ... )
  # future options "bootstrap","exact"
{

  retval <- list()

  # compute disequlibrium
  retval$diseq <- diseq(x)
  
  # compute confidence intervals
  retval$ci <- diseq.ci(x, R=ci.B, conf=conf)

  # do chisq test
  if(exact)
    retval$test  <- HWE.exact(x)
  else
    {
      tab <- retval$diseq$observed.no
      tab  <- 0.5 * (tab + t(tab))   # make symmetric for chisq.test
      retval$test  <- HWE.chisq(x, simulate.p.value=simulate.p.value,B=B,...)
    }

  
  retval$simulate.p.value <- simulate.p.value
  retval$B <- B
  retval$conf <- conf
  retval$ci.B <- ci.B
  retval$test$data.name  <- deparse(substitute(x))
  retval$call  <- match.call()
  class(retval)  <-  c("HWE.test")
  return(retval)
}



print.HWE.test  <-  function(x, show=c("D","D'","r","table"), ...)
  {

    cat("\n")
    cat("\t-----------------------------------\n")
    cat("\tTest for Hardy-Weinberg-Equilibrium\n")
    cat("\t-----------------------------------\n")
    cat("\n")
    if(!is.null(x$locus))
      {
        cat("\n")
        print( x$locus )
      }
    cat("Call: \n")
    print(x$call)
    cat("\n")
    if("D" %in% show)
      {
        cat("Raw Disequlibrium for each allele pair (D)\n")
        cat("\n") 
        print(x$diseq$D)
        cat("\n")
      }
    if("D'" %in% show)
      {
        cat("Scaled Disequlibrium for each allele pair (D')\n")
        cat("\n") 
        print(x$diseq$Dprime)
        cat("\n")
      }
    if("r" %in% show)
      {
        cat("Correlation coefficient for each allele pair (r)\n")
        cat("\n") 
        print(x$diseq$r)
        cat("\n")
      }
    if("table" %in% show)
      {
        cat("Observed vs Expected Allele Frequencies \n")
        cat("\n")
        print(x$diseq$table)
        cat("\n")
      }
    
    if( ncol(x$diseq$r) <= 2 )
      cat("Overall Values\n")
    else
      cat("Overall Values (mean absolute-value weighted by expected allele frequency)\n")
    cat("\n")

    show.tab <- NULL
    
    if("D" %in% show)
      show.tab <- rbind(show.tab, "  D"=x$diseq$D.overall)
    if("D'" %in% show)
      show.tab <- rbind(show.tab, "  D'"=x$diseq$Dprime.overall)
    if("r" %in% show)
      show.tab <- rbind(show.tab, "  r"=x$diseq$r.overall)

    colnames(show.tab) <- "Value"

    print(show.tab)
    
    cat("\n") 

    whichvec <- c("D","D'","r") %in% show

    cat("Confidence intervals computed via bootstrap using", x$ci.B, "samples\n")
    cat("\n")

    if(!is.null(x$ci$warning.text))
      cat(strwrap(paste("WARNING:", x$ci$warning.text), prefix="    * "),"\n",
          sep="\n")
    
    show.tab <- matrix(ncol=4, nrow=4)
    tmp <- format(x$ci$ci[,1:3], digits=getOption("digits"))
    show.tab[,1] <- tmp[,1]  # Observed
    show.tab[,2] <- paste("(", tmp[,2], ", ", tmp[,3], ")", sep="" )
    show.tab[,3] <- x$ci$ci[,4]
    show.tab[,4] <- ifelse(x$ci$ci[,5],"YES","*NO*")

    colnames(show.tab) <- c("Observed", "95% CI", "NA's", "Contains Zero?")
    rownames(show.tab) <- paste("  ", rownames(tmp), sep="")
    
    print(show.tab[whichvec,], quote=FALSE)

    cat("\n")
    cat("Significance Test:\n")
    print(x$test)
    cat("\n")
    cat("\n")
  }

HWE.test.data.frame <- function(x, ..., do.Allele.Freq=TRUE,
                                do.HWE.test=TRUE)
{

  data <- makeGenotypes(x)
  names <- names(data)[sapply(data, is.genotype)]
  
  for(i in names)
  {
    gene   <- getlocus(i)
    genedata <- data[[i]]
    
    cat("\n")
    cat("+-------------------------------------\n");
    if(!is.null(gene))
      {
        cat("|\tMarker:\t ")
        print(gene)
      }
    else
      cat("|\tMarker: ", i, "\n")
    cat("+-------------------------------------\n");
  
    if(do.Allele.Freq)
      {
         # compute and print the allele and genotype frequencies
        sum  <-  summary(genedata)
        print(sum)
      }
  
    if(do.HWE.test)
      {
        if(length(allele.names(genedata))<2)
          {
            cat( '*** No variant alleles observed, unable to perform\n',
                 '*** test for Hardy-Wienburg Equilibrium. \n', sep='')
          }
        else
          {
            # now do and print the HWE test
            hwe  <- HWE.test(genedata, ...)
            print(hwe)
          }
  
      }
  }
  
}

# $Id: LD.R 453 2005-11-09 17:04:02Z warnes $

# R translation of Cathy Stack's SAS macro
# Assumes 2-alleles

LD <- function(g1,...)
  UseMethod("LD",g1)

LD.data.frame <- function(g1,...)
  {
    gvars <- sapply( g1, function(x) (is.genotype(x) && nallele(x)==2) )
    if(any(gvars==FALSE))
      {
        warning("Non-genotype variables or genotype variables ",
                "with more or less than two alleles detected. ",
                "These variables will be omitted: ",                
                paste( colnames(g1)[!gvars] , collapse=", " )
                )
        g1 <- g1[,gvars]
      }
    

    P <- matrix(nrow=ncol(g1),ncol=ncol(g1))
    rownames(P) <- colnames(g1)
    colnames(P) <- colnames(g1)

    P <- D <- Dprime <- nobs <- chisq <- p.value <- corr <- R.2 <- P

    for(i in 1:(ncol(g1)-1) )
      for(j in (i+1):ncol(g1) )
        {
          ld <- LD( g1[,i], g1[,j] )
          
          D      [i,j] <- ld$D
          Dprime [i,j] <- ld$"D'"
          corr   [i,j] <- ld$"r"
          R.2    [i,j] <- ld$"R^2"          
          nobs   [i,j] <- ld$"n"
          chisq  [i,j] <- ld$"X^2"
          p.value[i,j] <- ld$"P-value"
        }
    
    retval <- list(
                   call=match.call(),
                   "D"=D,
                   "D'"=Dprime,
                   "r" = corr,
                   "R^2" = R.2,
                   "n"=nobs,
                   "X^2"=chisq,
                   "P-value"=p.value
           )

    class(retval) <- "LD.data.frame"
    
    retval
  }

LD.genotype <- function(g1,g2,...)
  {
    if(is.haplotype(g1) || is.haplotype(g2))
      stop("Haplotype options are not yet supported.")
    
    if(nallele(g1)!=2 || nallele(g2)!=2)
      stop("This function currently only supports 2-allele genotypes.")

    prop.A <- summary(g1)$allele.freq[,2]
    prop.B <- summary(g2)$allele.freq[,2]
    
    major.A <- names(prop.A)[which.max(prop.A)]
    major.B <- names(prop.B)[which.max(prop.B)]
    pA <- max(prop.A, na.rm=TRUE)
    pB <- max(prop.B, na.rm=TRUE)
    pa <- 1-pA
    pb <- 1-pB

    Dmin <- max(-pA*pB, -pa*pb)
    pmin <- pA*pB + Dmin;

    Dmax <- min(pA*pb, pB*pa);
    pmax <- pA*pB + Dmax;

    counts <- table(
                    allele.count(g1, major.A),
                    allele.count(g2, major.B)
                    )

    n3x3 <- matrix(0, nrow=3, ncol=3)
    colnames(n3x3) <- rownames(n3x3) <- 0:2

    # ensure the matrix is 3x3, with highest frequency values in upper left
    for(i in rownames(counts))
      for(j in colnames(counts))
        n3x3[3-as.numeric(i),3-as.numeric(j)] <- counts[i,j]

    
    loglik <- function(pAB,...)
      {
        (2*n3x3[1,1]+n3x3[1,2]+n3x3[2,1])*log(pAB) +
        (2*n3x3[1,3]+n3x3[1,2]+n3x3[2,3])*log(pA-pAB) +
        (2*n3x3[3,1]+n3x3[2,1]+n3x3[3,2])*log(pB-pAB) +
        (2*n3x3[3,3]+n3x3[3,2]+n3x3[2,3])*log(1-pA-pB+pAB) + 
        n3x3[2,2]*log(pAB*(1-pA-pB+pAB) + (pA-pAB)*(pB-pAB))
      }

    # SAS code uses:
    #
    #s <- seq(pmin+0.0001,pmax-0.0001,by=0.0001)
    #lldmx <- loglik(s)
    #maxi <- which.max(lldmx)
    #pAB <- s[maxi]

    # but this should be faster:
    solution <- optimize(
                         loglik,
                         lower=pmin+.Machine$double.eps,
                         upper=pmax-.Machine$double.eps,
                         maximum=TRUE
                         )
    pAB <- solution$maximum

    estD <- pAB - pA*pB
    if (estD>0)  
      estDp <- estD / Dmax
    else
      estDp <- estD / Dmin

    n <-  sum(n3x3)

    corr <- estD / sqrt( pA * pB * pa * pb )
    
    dchi <- (2*n*estD^2)/(pA * pa * pB* pb)
    dpval <- 1 - pchisq(dchi,1)

    retval <- list(
                   call=match.call(),
                   "D"=estD,
                   "D'"=estDp,
                   "r" = corr,
                   "R^2" = corr^2,
                   "n"=n,
                   "X^2"=dchi,
                   "P-value"=dpval
                   )

    class(retval) <- "LD"
    
    retval
    
  }

# $Id: HWE.chisq.R 1352 2012-08-14 14:21:35Z warnes $


###
### Hardy-Weinberg Equilibrium Significance Test
###


HWE.chisq <- function(x, ...)
  UseMethod("HWE.chisq")

HWE.chisq.genotype <- function (x, simulate.p.value = TRUE, B = 10000, ...)
{
    observed.no <- table(factor(allele(x, 1), levels = allele.names(x)), 
        factor(allele(x, 2), levels = allele.names(x)))
    tab <- observed.no
    tab <- 0.5 * (tab + t(tab))
    k <- ncol(tab)
    if(simulate.p.value)
      {
        test <- chisq.test(tab, simulate.p.value = simulate.p.value, 
                           B = B, ...)
      }
    else
      {
        test <- chisq.test(tab, ...)
        test$parameter <- k*(k-1)/2
        test$p.value <- pchisq(test$statistic, test$parameter, lower.tail = FALSE)
        names(test$statistic) <- "X-squared"
        names(test$parameter) <- "df"
      }
    return(test)
}

### order.genotype.R
###------------------------------------------------------------------------
### What: Get order of genotype object according to order of allele names or
###       genotype names
### Time-stamp: <2007-07-20 10:47:44 ggorjan>
###------------------------------------------------------------------------

## Define order as generic function
order <- function(..., na.last=TRUE, decreasing=FALSE)
  UseMethod("order")

## Get default method for order from base package
order.default <- function(..., na.last=TRUE, decreasing=FALSE)
  base::order(..., na.last=na.last, decreasing=decreasing)

## Genotype/Haplotype methods
order.genotype <- function(..., na.last=TRUE, decreasing=FALSE,
                           alleleOrder=allele.names(x), genotypeOrder=NULL)
{
  x <- list(...)[[1]]
  isHap <- is.haplotype(x)
  reorder <- ifelse(isHap, "no", "yes")
  if (is.null(genotypeOrder)) {

    ## --- Sort by alleleOrder ---

    alleleOrder <- unique(alleleOrder)

    ## Prepair working genotype data for given alleles only
    if(!identical(alleleOrder, allele.names(x)))
      x <- genotype(x, alleles=alleleOrder, reorder=reorder)

    ## Prepair working data for sort i.e. 01_allele1/15_allele15
    tmp <- as.character(x)
    id <- seq(along=alleleOrder)
    id <- formatC(id, width=ceiling(log10(max(id))), flag="0")
    for(i in seq(along=alleleOrder)) {
      tmp <- sub(pattern=paste(alleleOrder[i], "/", sep=""),
                 replacement=paste(i, "_", alleleOrder[i], "/", sep=""),
                 x=tmp)
      tmp <- sub(pattern=paste("/", alleleOrder[i], sep=""),
                 replacement=paste("/", i, "_", alleleOrder[i], sep=""),
                 x=tmp)
    }
  } else {

    ## --- Sort by genotypeOrder ---

    genotypeOrder <- unique(genotypeOrder)

    if(!isHap) {
      ## Match both A/B and B/A
      genotypeOrder <- .genotype2Haplotype(x=genotypeOrder)
    }
    tmp <- match(x, genotypeOrder)
  }
  ## print(tmp)
  return(order(tmp, na.last=TRUE, decreasing=FALSE))
}

sort.genotype <- function(x, decreasing=FALSE, na.last=NA, ...,
                          alleleOrder=allele.names(x), genotypeOrder=NULL)
{
  x[order(x, decreasing=decreasing, na.last=na.last,
          alleleOrder=alleleOrder, genotypeOrder=genotypeOrder)]
}

## No need for haplotype methods as they are exactly the same and haplotype
## is extended class of genotype sort.haplotype

###------------------------------------------------------------------------
### order.genotype.R ends here

write.pop.file <- function(data,
                           file="",
                           digits=2,
                           description="Data from R"
                           )
  {
    which <- sapply(data, is.genotype)
    if(!all(which)) warning("Data contianed non-genotype variables.",
                            " These have been ignored: ",
                            paste(colnames(data)[!which]) )

    data <- data[,which]

    # convert allele names into two or three digit numbers
    for( col in names(data) )
      {
        # first convert to numbers
        a1 <- as.numeric(factor(allele(data[[col]],1)))
        a2 <- as.numeric(factor(allele(data[[col]],2)))

        # convert NA to 0
        a1[is.na(a1)] <- 0
        a2[is.na(a2)] <- 0
        
        # now format to have correct # of digits
        a1 <- formatC( a1, width=digits, flag="0")
        a2 <- formatC( a2, width=digits, flag="0")
        
        # now paste back together
        data[[col]] <- paste(a1,a2,sep="")
      }

    if(file=="")
      f <- stdout()
    else
      f <- file(file,"w")

    # header line
    cat(description, file=f)
    cat("\n", file=f)

    # marker names
    cat(colnames(data),sep=" ", file=f)
    cat("\n", file=f)

    # group token
    cat("POP", file=f)
    cat("\n", file=f)

    # write allele data.  First token is row id, followed by a comma
    # markers are separated by space
    rownames(data) <- paste(rownames(data),",", sep="")
    write.table( data, file=f, sep=" ", quote=FALSE, col.names=F)

    if(file!="")
      close(f)
    
  }

# $Id: gregorius.R 114 2003-05-22 17:25:23Z warnesgr $
#
# Code contributed by David Duffy <davidD@qumr.edu.au>.
#
# Gregorius, H.-R. 1980. The probability of losing an allele when
# diploid genotypes are sampled.  Biometrics 36, 643-652.
#
# Formula from "Corollary 2" and "Corollary 3" of that paper
#
# N is the number of genotypes sampled,
# freq=frequency of least common allele to be detected by the study,
# missprob=the probability of missing at least one allele
#
# tol=smallest term in series to be accumulated
#
gregorius <- function(freq, N, missprob, tol=1.0e-10, maxN=1e4, maxiter=100,
                      showiter=FALSE)
{
   

  find.alpha <- function(N, freq, tol) #, showiter=FALSE)
    {
      n<- floor(1/freq)
      i<-1
      sgn<- -1
      term<-1.0
      res<-0.0
      while(abs(term)>tol && i<n) {
        sgn<- (-1) ^ (i+1)
        term<- exp( lchoose(n-1,i) +
                   log(exp(N*log(1-i*freq))+
                       exp(i+N*log(freq)+(N-1)*log(n-i))))
        res<-res+sgn*term
        i<-i+1

#        if(showiter)
#          {
#            cat("i=",i,"\n")
#            cat("sgn=",sgn,"\n")
#            cat("term=",term,"\n")
#            cat("res=",res,"\n")
#          }
      }

      max(min(res,1),0)
    }


  retval <- list()
  retval$call <- match.call()
    
  
  if(!missing(N) && missing(missprob) )
    {
      retval$method <- "Compute missprob given N and freq"
      retval$freq <- freq
      retval$N <- N
      retval$missprob <- find.alpha(N=N,freq=freq,tol=tol) 
    }
  else if(missing(N) && !missing(missprob) )
    {
      retval$method <- "Determine minimal N given missprob and freq"
      retval$freq <- freq
      val <- binsearch( function(N) find.alpha(N=N, freq=freq, tol=tol),
                       range=c(1, maxN), target=missprob, showiter=showiter,
                       maxiter=maxiter )
      if(length(val$where)==2)
        {
          retval$N <- val$where[2]
          retval$missprob <- val$value[2]
        }
      else
        {
          retval$N <- val$where[1]
          retval$missprob <- val$value[1] 
       }
    }
  else
    stop("Exactly two of N, freq, and missprob must be specified")

  return(retval)
}

diseq <- function(x, ...)
{
	UseMethod("diseq")
}

diseq.genotype <- function(x, ...)
  {
    if (nallele(x) < 2)
      {
        warning("Only 1 Marker allele. Returning NA")
        return(NA)
      }

    observed.no <- table( factor(allele(x,1), levels=allele.names(x)),
                          factor(allele(x,2), levels=allele.names(x)) )
    observed <- prop.table(observed.no)
    observed <- 1/2 * (observed + t(observed) )

    retval <- diseq.table(observed)
    retval$observed.no <- observed.no
    retval$call <- match.call()
    retval
  }

diseq.table <- function(x, ...)
{
  observed <- x
  allele.freq <- apply(observed,1,sum)
  # equal to: allele.freq <- apply(observed,2,sum)

  expected <- outer(allele.freq, allele.freq, "*")

  oeTab <- cbind(Obs=c(observed),
                 Exp=c(expected),
                 "Obs-Exp"=c(observed - expected))
  rownames(oeTab) <- outer(rownames(observed),
                           rownames(observed), paste, sep="/")

  diseq <- observed - expected
  diag(diseq) <- NA

  dmax.positive <- expected
  # equals: max( p(i)p(j), p(j)p(i) )

  dmax.negative <- outer(allele.freq, allele.freq, pmin ) - expected
  # equals: min( p(i) * (1 - p(j)), p(j)( 1 - (1-p(i) ) ) )

  dprime <- diseq / ifelse( diseq > 0, dmax.positive, dmax.negative )

  # r gives the pairwise correlation coefficient for pairs containing at lease
  # one allele from the specified pair.
  # For two alleles:
  #    corr coefficient = diseq / sqrt( p(a) * (1-p(a) ) * p(b) * (1-p(b)) )
  #p.1.minus.p <- allele.freq * (1-allele.freq)
  #r <-  -diseq  / sqrt( outer( p.1.minus.p, p.1.minus.p, "*") )
  r.denom <- sqrt(    allele.freq  %*% t(  allele.freq)) *
             sqrt( (1-allele.freq) %*% t(1-allele.freq))
  r <- -diseq  / r.denom

  # above formula works unchanged for 2 alleles, but requires adjustment
  # for multiple alleles.
  # r <- r * (length(allele.freq) - 1)

  offdiag.expected <- expected
  diag(offdiag.expected) <- NA
  sum.expected <- sum(offdiag.expected, na.rm=TRUE)

  if(all(dim(x)==2)) # 2 allele case
    {
      diseq.overall <- diseq[1,2]
      dprime.overall <- dprime[1,2]
      r.overall <- r[1,2]
      R2.overall <- r.overall^2
    }
  else
    {
      diseq.overall <- sum( abs(diseq) * expected , na.rm=TRUE ) / sum.expected
      dprime.overall <- sum( abs(dprime) * expected , na.rm=TRUE ) / sum.expected
      r.overall <- sum( abs(r) * expected , na.rm=TRUE ) / sum.expected
      R2.overall <- r.overall^2
    }

  #diag(r) <- 1.0

  retval <- list(
                 call = match.call(),
                 observed=observed,
                 expected=expected,
                 table=oeTab,
                 allele.freq=allele.freq,
                 D=diseq,
                 Dprime=dprime,
                 r=r,
                 R2=r^2,
                 D.overall=diseq.overall,
                 Dprime.overall=dprime.overall,
                 r.overall = r.overall,
                 R2.overall = R2.overall
                 )

  class(retval) <- "diseq"
  retval
}

print.diseq  <-  function(x, show=c("D","D'","r","R^2","table"), ...)
  {

    cat("\n")
    if(!is.null(x$locus))
      {
        cat("\n")
        print( x$locus )
      }
    cat("\n")
    cat("Call: \n")
    print(x$call)
    cat("\n")
    if("D" %in% show)
      {
        cat("Disequlibrium for each allele pair (D)\n")
        cat("\n")
        print(x$D)
        cat("\n")
      }
    if("D'" %in% show)
      {
        cat("Disequlibrium for each allele pair (D')\n")
        cat("\n")
        print(x$Dprime)
        cat("\n")
      }
    if("r" %in% show)
      {
        cat("Correlation coefficient for each allele pair (r)\n")
        cat("\n")
        print(x$r)
        cat("\n")
      }
    if("R^2" %in% show)
      {
        cat("R^2 for each allele pair\n")
        cat("\n")
        print(x$R2)
        cat("\n")
      }
    if("table" %in% show)
      {
        cat("Observed vs Expected frequency table\n")
        cat("\n")
        print(x$table)
        cat("\n")
      }

    if( any(c("D","D'","r") %in% show))
      {
        if( ncol(x$r) <= 2 )
          cat("Overall Values\n")
        else
          cat("Overall Values (mean absolute-value weighted by expected allele frequency)\n")
        cat("\n")

        if("D" %in% show)
          cat("  D  :  ", x$D.overall, "\n", sep="")
        if("D'" %in% show)
          cat("  D' :  ", x$Dprime.overall, "\n", sep="")
        if("r" %in% show)
          cat("  r  :  ", x$r.overall, "\n", sep="")
        if("R^2" %in% show)
          cat("  R^2:  ", x$R2.overall, "\n", sep="")
        cat("\n")
      }

    cat("\n")
  }

diseq.ci <- function(x, R=1000, conf=0.95, correct=TRUE, na.rm=TRUE, ...)
{
  if (!("genotype") %in% class(x) )
    stop("x must inherit from class 'genotype'.")

  if( any(is.na(x) ) )
    {
      if( na.rm)
        x <- na.omit(x)
      else
        stop("Missing values and NaN's not allowed if `na.rm' is FALSE.")
    }

  # step 1 - generate summary table
  observed.no <- table( factor(allele(x,1), levels=allele.names(x)),
                        factor(allele(x,2), levels=allele.names(x)) )
  observed <- prop.table(observed.no)
  observed <- 1/2 * (observed + t(observed) )

  # step 2 - make table into a probability vector for calling rmultinom
  n <- sum(observed.no)
  prob.vector <- c(observed)

  # step 3 - sample R multinomials with the specified frequenceis
  # (include observed data to avoid bias)
  resample.data <- cbind(c(observed.no),
                         rmultz2( n, prob.vector, R ) )

  bootfun <- function(x) {
    observed[,] <- x/n
    observed <- 1/2 * (observed + t(observed) )
    d <-  diseq(observed)
    c( "Overall D  "=d$D.overall,
       "Overall D' "=d$Dprime.overall,
       "Overall r  "=d$r.overall,
       "Overall R^2"=d$R2.overall)
  }

  results <- apply( resample.data, 2, bootfun )

  alpha.2 <- (1-conf)/2

#  ci <- t(apply(results, 1,
#              quantile, c( alpha.2 , 1-alpha.2), na.rm=TRUE ))

  if(length(allele.names(x))<=2)
    {
      ci <- t(apply(results, 1, function(x) quantile(x, c(0.025, 0.975),
                                                     na.rm=na.rm ) ) )

      warning.text <- paste("The R^2 disequlibrium statistics is bounded",
                            "between [0,1].  The confidence ",
                            "intervals for R^2 values near 0 and 1 are",
                            "ill-behaved.", sep=" ")

      if(correct)
        {
          warning.text <- paste(warning.text, "A rough correction has",
                                "been applied, but the intervals still",
                                "may not be correct for R^2 values near",
                                "0 or 1.",
                                sep=" ")

          X <- results["Overall R^2",]
          ci["Overall R^2",] <- ci.balance(X,X[1],confidence=conf,
                                           minval=0,maxval=1)$ci
        }
    }
  else
    {
      warning.text <- paste("For more than two alleles, overall",
                            "disequlibrium statistics are bounded",
                            "between [0,1].  Because of this, confidence",
                            "intervals for values near 0 and 1 are",
                            "ill-behaved.", sep=" ")

      if(correct)
        {
          warning.text <- paste(warning.text, "A rough correction has been applied, but",
                                "the intervals still may not be correct for values near 0 or 1.",
                                sep=" ")

          ci <- t(apply(results, 1,
                        function(x)
                        ci.balance(x,x[1],confidence=conf,
                                   minval=0,maxval=1)$ci ))
        }
      else
        ci <- t(apply(results, 1, function(x) quantile(x, c(0.025, 0.975) ) ) )

      warning(paste(strwrap(c(warning.text,"\n"),prefix="  "),collapse="\n") )
    }

  na.count <-  function(x) sum(is.na(x))
  nas <- apply( results, 1, na.count)

  zero.in.range <- (ci[,1] <= 0) & (ci[,2] >= 0)

  ci <- cbind( "Observed"=results[,1], ci, "NAs"=nas,
               "Zero in Range"=zero.in.range )

  outside.ci <- (ci[,1] < ci[,2]) | (ci[,1] > ci[,3])

  if( any(outside.ci) )
    warning("One or more observed value outide of confidence interval. Check results.")

  if(any(nas>0))
    warning("NAs returned from diseq call")


  retval <- list(
         call=match.call(),
         R=R,
         conf=conf,
         ci=ci,
         warning.text=warning.text
         )

  retval
}

# $Id: summary.genotype.R 1314 2007-09-12 10:41:11Z ggorjan $

###
### Provide the frequency and proportions of alleles and genotypes
###

# used when summary.genotype is called from summary.data.frame:
shortsummary.genotype <- function(object, ..., maxsum)
  {
    tmp <- summary.factor(object, maxsum=maxsum)
    retval <- paste(format(tmp), " (", format(round(prop.table(tmp)*100)), "%)", sep='' )
    names(retval) <- names(tmp)
    #retval <- retval[order(tmp, decreasing=TRUE)]
    retval
  }

# general function
summary.genotype  <-  function(object,...,maxsum)
  {
    # if we are called from within summary.data.frame, fall back to
    # summary.factor so that we don't mess up the display
    if(!missing(maxsum))
      return(shortsummary.genotype(object,...,maxsum=maxsum))

    retval  <-  list()
#    retval$genotype  <- object
    retval$allele.names  <- allele.names(object)

    retval$locus  <- attr(object,"locus")
    class(retval)  <- "summary.genotype"

    af  <- table(allele(object))

    # make sure af has same order as allele.names...
    #
    missed <- !names(af) %in% retval$allele.names 
    af.tab <- rep(0,length(retval$allele.names))
    names(af.tab) <- retval$allele.names
    af.tab[names(af)] <- af
    #

    paf <- prop.table(af.tab)
    retval$allele.freq    <- cbind("Count"=af.tab,"Proportion"=paf)

    gf  <- table( object )
    pgf <- prop.table(gf)
    retval$genotype.freq  <- cbind("Count"=gf,"Proportion"=pgf)

    ## Sort by genotypeOrder
    asFun <- as.genotype
    if(is.haplotype(object)) asFun <- as.haplotype
    tmp <- asFun(rownames(retval$genotype.freq),
                 alleles=allele.names(object))
    tmp <- order(tmp, genotypeOrder=genotypeOrder(object))
    retval$genotype.freq <- retval$genotype.freq[tmp, ]

    ### from code submitted by David Duffy <davidD@qimr.edu.au>
    #
    n.typed<-sum(gf)
    correction<-2*n.typed/max(1,2*n.typed-1)
    ehet<-(1-sum(paf*paf))
    matings<- (paf %*% t(paf))^2
    uninf.mating.freq <- sum(matings)-sum(diag(matings))
    pic<- ehet - uninf.mating.freq

    retval$Hu <- correction * ehet
    retval$pic <- pic
    retval$n.typed <- n.typed
    retval$n.total <- length(object)
    retval$nallele <- nallele(object)
    #
    ###

    ## Add info on NA values
    if(any(is.na(object))){
        retval$allele.freq    <- rbind(retval$allele.freq,
                                       "NA"=c(sum(is.na(allele(object))),NA))
        retval$genotype.freq  <- rbind(retval$genotype.freq,
                                       "NA"=c(sum(is.na(object)),NA))
    }

    return(retval)
  }

print.summary.genotype  <-  function(x,...,round=2)
  { 
    if(!is.null(x$locus))
      {
        cat("\n")
        print( x$locus )
      }

    cat("\n")
    cat("Number of samples typed: ", x$n.typed,
      " (", round(100*x$n.typed/x$n.total,1), "%)\n", sep="")

    cat("\n")
    cat("Allele Frequency: (", x$nallele, " alleles)\n", sep="")
    print(round(x$allele.freq,digits=round),...)
    cat("\n")


    cat("\n")
    cat("Genotype Frequency:\n")
    print(round(x$genotype.freq,digits=round),...)
    cat("\n")

    cat("Heterozygosity (Hu)  = ",  x$Hu, "\n", sep="")
    cat("Poly. Inf. Content   = ",  x$pic, "\n", sep="")
    cat("\n")

    invisible(x)
  }

### groupGenotype.R
###------------------------------------------------------------------------
### What: Group genotype values code
### Time-stamp: <2007-07-21 12:04:16 ggorjan>
###------------------------------------------------------------------------

groupGenotype <- function(x, map, haplotype=FALSE, factor=TRUE,
                          levels=NULL, verbose=FALSE)
{
  if(!is.genotype(x))
    stop("'x' must be of a genotype or haplotype class")
  if(any(names(map) == ""))
    stop("all list components in 'map' need to have a name")

  alleles <- allele.names(x)

  ## Put else at the end and change it to */*
  elseF <- FALSE
  elsePos <- sapply(map, function(x) length(x) == 1 && x == ".else")
  if(any(elsePos)) {
    elseF <- TRUE
    map <- c(map[!elsePos], map[elsePos])
    map[elsePos] <- "*/*"
  }

  ## Extend the map
  for(i in seq(along=map)) {
    map[[i]] <- unlist(.matchGenotype(alleles=alleles,
                                                 pattern=map[[i]],
                                                 haplotype=haplotype),
                       use.names=FALSE)
  }

  ## Remove duplicates sequentially over all map
  nM <- length(map)
  if(nM > 1) {
    for(i in 2:nM) {
      test <- map[[i]] %in% unlist(map[1:(i - 1)], use.names=FALSE)
      map[[i]] <- map[[i]][!test]
    }
  }

  ## Show matches
  if(verbose) print(map)

  ## Group
  x <- as.factor(x)
  if(!is.null(levels)) {
    if(length(map) != length(levels))
      warning("length of 'map' and 'levels' does not match")
    map <- map[levels]
  }
  mapLevels(x) <- as.levelsMap(map)

  ## Factor?
  if(!factor) x <- as.character(x)

  ## Return
  x
}

.matchGenotype <- function(x, alleles=allele.names(x), pattern, haplotype=FALSE)
{
  ## Internal function
  ##
  ## Finds genotype matches according to given patterns out of possible
  ## genotype values that might appear in genotype data
  ##
  ## Arguments:
  ## x - genotype or haplotype
  ## alleles, character, allele names
  ## pattern - character, pattern in form of "A/A", "A/B", "A/*", "*/A" or "*/*"
  ## haplotype - logical, should order of alleles in the pattern matter
  ##
  ## Value:
  ## A list of length equal to length of pattern values. Each list
  ## component is named with pattern and has genotype values that match a
  ## pattern
  ##
  ## Details:
  ## Internally, \code{\link{genotype}} can store values as "A/B" or "B/A",
  ## so output for pattern="A/*" holds both "A/B" and "B/A", when
  ## haplotype=FALSE and there are two alleles (A and B).
  ##
  ## Example:
  ## pattern <- c("A/*", "A/B", "*/*", "B/A")
  ## .matchGenotype(alleles=c("B", "A"), pattern=pattern)
  ## $`A/*`
  ## [1] "A/B" "B/A" "A/A"
  ## $`A/B`
  ## [1] "A/B" "B/A"
  ## $`*/*`
  ## [1] "B/B" "B/A" "A/B" "A/A"
  ## $`B/A`
  ## [1] "B/A" "A/B"
  ##
  ## .matchGenotype(alleles=c("B", "A"), pattern=pattern, haplotype=TRUE)
  ## $`A/*`
  ## [1] "A/B" "A/A"
  ## $`A/B`
  ## [1] "A/B"
  ## $`*/*`
  ## [1] "B/B" "B/A" "A/B" "A/A"
  ## $`B/A`
  ## [1] "B/A"

  if(!missing(x)) {
    if(!is.genotype(x))
      stop("'x' must be of a genotype or haplotype class")
  } else {
    if(missing(alleles))
      stop("at least one of 'x' or 'alleles' must be given")
  }
  if(missing(pattern))
    stop("'pattern' must be given")

  nP <- length(pattern)
  ret <- vector(mode="list", length=nP)
  names(ret) <- pattern

  ## Change * with allele names setup
  parts <- .genotype2Allele(x=pattern)
  parts <- cbind(parts, 1:nP)
  testStar <- parts == "*"
  nA <- length(alleles)

  ## Expand A/* to A/{alleles} etc.
  for(i in 1:nrow(parts)) {
    ## Allele beside *
    whichStar <- which(!testStar[i, 1:2])
    nWS <- length(whichStar)
    if(nWS < 2) {
      if(nWS == 1) { # A/* or */A
        a <- parts[i, whichStar]
        ## Create possible genotypes
        if(whichStar == 1) {
          parts <- rbind(parts, cbind(a, alleles, i))
        } else {
          parts <- rbind(parts, cbind(alleles, a, i))
        }
      } else {       # */*
        tmp <- expectedHaplotypes(alleles=alleles)
        tmp <- .genotype2Allele(x=tmp)
        parts <- rbind(parts, cbind(tmp, i))
      }
    }
  }

  ## Remove *
  testStar <- rowSums(parts == "*") > 0
  parts <- parts[!testStar, , drop=FALSE]

  ## Order by pattern and create genotypes
  parts <- parts[order(parts[, 3, drop=FALSE]), , drop=FALSE]
  parts <- cbind(paste(parts[, 1], parts[, 2], sep="/"), parts[, 3])

  ## Fill the return
  patternId <- unique(parts[, 2])
  for(i in 1:nP) {
    ret[[i]] <- parts[parts[, 2] == patternId[i], 1]
  }

  ## For genotype treat A/* the same as */A and A/B as B/A
  if(!haplotype) ret <- lapply(ret, .genotype2Haplotype)

  ## Return
  ret
}

###------------------------------------------------------------------------
### groupGenotype.R ends here

# $Id: makeGenotypes.R 1340 2008-08-20 19:04:32Z warnes $

#
# convert all genotype-compatible variables in a dataframe to genotypes
#
makeGenotypes <- function( data, convert, sep="/", tol=0.5, ...,
                           method=as.genotype )
                          
  {
    data <- as.data.frame(data)
    
    if(missing(convert))
      {
        fun <- function(x) length(unlist(grep(sep, as.character(x) )))
        convert <- sapply( data,  fun )/nrow(data) > tol
      }

    #cat("Convert:");print(convert);

    if(is.list(convert))
      {
        if( !all(sapply(convert,length)==2) )
          stop("When convert is a list, each element must be a 2-vector.")

        namelist <- names(data)
        
        for(pair in convert)
           {
             if(!is.character(pair))
               pair <- namelist[pair]
             # replace first column in pair with new data,
             index <- which(colnames(data)==pair[1])
             data[[ index ]] <- method(data[[ pair[1] ]],
                                       data[[ pair[2] ]], sep=sep,
                                                ... )
             colnames(data)[index] <- paste(pair,collapse=sep)
             data[[ pair[1] ]] <- data[[ pair[2] ]] <- NULL
           }
      }
    else
      {
        if(is.character(convert))
          namelist <- convert
        else
          namelist <- colnames(data)[convert]

        for(col in namelist)
          data[[col]] <- method(data[[col]], sep=sep, ... )
      }

    data
}

makeHaplotypes <- function( data, convert, sep="/", tol=0.9, ... )
  {
    makeGenotypes( data=data, convert=convert, sep=sep, tol=tol,
                  method=as.haplotype, ... )    
  }

% $Id: genotype.Rd 1353 2012-08-14 14:23:17Z warnes $

\name{genotype}
\alias{genotype}
\alias{haplotype}
\alias{is.genotype}
\alias{is.haplotype}
\alias{as.genotype}
\alias{as.haplotype}

%%
\alias{print.genotype}
\alias{==.genotype}
\alias{==.haplotype}
\alias{\%in\%}
\alias{\%in\%.default}
\alias{\%in\%.genotype}
\alias{\%in\%.haplotype}
\alias{[.genotype}
\alias{[.haplotype}
\alias{[<-.genotype}
\alias{[<-.haplotype}
\alias{heterozygote.genotype}
\alias{homozygote.genotype}
\alias{print.allele.count}
\alias{print.allele.genotype}
\alias{allele.count.genotype}
%\alias{allele.genotype}
%\alias{allele.names}
\alias{as.genotype.allele.count}
\alias{as.genotype.character}
\alias{as.genotype.default}
\alias{as.genotype.factor}
\alias{as.genotype.genotype}
\alias{as.genotype.haplotype}
\alias{as.genotype.table}
\alias{nallele}

\title{Genotype or Haplotype Objects.}
\description{
  \code{genotype} creates a genotype object.

  \code{haplotype} creates a haplotype object.

  \code{is.genotype} returns \code{TRUE} if \code{x} is of class
  \code{genotype}

  \code{is.haplotype} returns \code{TRUE} if \code{x} is of class
  \code{haplotype}

  \code{as.genotype} attempts to coerce its argument into an object of
  class \code{genotype}.

  \code{as.genotype.allele.count} converts allele counts (0,1,2) into
  genotype pairs ("A/A", "A/B", "B/B").

  \code{as.haplotype} attempts to coerce its argument into an object of
  class \code{haplotype}.

  \code{nallele} returns the number of alleles in an object of class
  \code{genotype}.

}
\usage{
  genotype(a1, a2=NULL, alleles=NULL, sep="/", remove.spaces=TRUE,
           reorder = c("yes", "no", "default", "ascii", "freq"),
           allow.partial.missing=FALSE, locus=NULL,
           genotypeOrder=NULL)

  haplotype(a1, a2=NULL, alleles=NULL, sep="/", remove.spaces=TRUE,
            reorder="no", allow.partial.missing=FALSE, locus=NULL,
            genotypeOrder=NULL)

  is.genotype(x)

  is.haplotype(x)

  as.genotype(x, ...)

  \method{as.genotype}{allele.count}(x, alleles=c("A","B"), \dots )

  as.haplotype(x, ...)

  \method{print}{genotype}(x, \dots)

  nallele(x)
}

\arguments{
  \item{x}{ either an object of class \code{genotype} or
    \code{haplotype} or an object to be converted to class \code{genotype} or
    \code{haplotype}.}
  \item{a1,a2}{ vector(s) or matrix containing two alleles
    for each individual. See details, below.}
  \item{alleles}{ names (and order if \code{reorder="yes"}) of possible
    alleles.}
  \item{sep}{ character separator or column number used to divide
    alleles when \code{a1} is a vector of strings where each string
    holds both alleles. See below for details.}
  \item{remove.spaces}{ logical indicating whether spaces and tabs will
    be removed from a1 and a2  before processing.}
  \item{reorder}{how should alleles within an individual be reordered.
    If \code{reorder="no"}, use the order specified by the alleles
    parameter.  If \code{reorder="freq"} or
    \code{reorder="yes"}, sort alleles within each individual by
    observed frequency.  If \code{reorder="ascii"}, reorder alleles in
    ASCII order (alphabetical, with all upper case before
    lower case). The default value for \code{genotype}
    is \code{"freq"}.  The default value for \code{haplotype} is
    \code{"no"}.
  }
  \item{allow.partial.missing}{logical indicating whether one allele is
    permitted to be missing.  When set to \code{FALSE} both alleles
    are set to \code{NA} when either is missing.}
  \item{locus}{ object of class locus, gene, or marker, holding
    information about the source of this genotype.}
  \item{genotypeOrder}{character, vector of genotype/haplotype names so
    that further functions can sort genotypes/haplotypes in wanted
    order}
  \item{...}{optional arguments}
}

\details{

  Genotype objects hold information on which gene or marker alleles were
  observed for different individuals.  For each individual, two alleles
  are recorded.

  The genotype class considers the stored alleles to be unordered, i.e., "C/T"
  is equivalent to "T/C".  The haplotype class considers the order of the
  alleles to be significant so that "C/T" is distinct from "T/C".

  When calling \code{genotype} or \code{haplotype}:

  \itemize{
  \item If only \code{a1} is provided and is a character vector, it is
    assumed that each element encodes both alleles. In this case, if
    \code{sep} is a character string, \code{a1} is assumed to be coded
    as "Allele1<sep>Allele2".  If \code{sep} is a numeric value, it is
    assumed that character locations \code{1:sep} contain allele 1 and
    that remaining locations contain allele 2.

  \item If \code{a1} is a matrix, it is assumed that column 1 contains
  allele 1 and column 2 contains allele 2.

  \item If \code{a1} and \code{a2} are both provided, each is assumed to
  contain one allele value so that the genotype for an individual is
  obtained by \code{paste(a1,a2,sep="/")}.

  }

  If \code{remove.spaces} is TRUE, (the default) any whitespace
  contained in \code{a1} and \code{a2} is removed when the genotypes are
  created.  If whitespace is used as the separator, (eg "C C", "C T",
  ...), be sure to set remove.spaces to FALSE.

  When the alleles are explicitly specified using the \code{alleles}
  argument, all potential alleles not present in the list will be
  converted to \code{NA}.

  NOTE: \code{genotype} assumes that the order of the alleles is not important
  (E.G., "A/C" == "C/A").  Use class \code{haplotype} if order is significant.

  If \code{genotypeOrder=NULL} (the default setting), then
  \code{\link{expectedGenotypes}} is used to get standard sorting order.
  Only unique values in \code{genotypeOrder} are used, which in turns
  means that the first occurrence prevails. When \code{genotypeOrder} is
  given some genotype names, but not all that appear in the data, the
  rest (those in the data and possible combinations based on allele
  variants) is automatically added at the end of
  \code{genotypeOrder}. This puts "missing" genotype names at the end of
  sort order. This feature is especially useful when there are a lot of
  allele variants and especially in haplotypes. See examples.

}

\value{

  The genotype class extends "factor" and haplotype extends
  genotype. Both classes have the following attributes:
  \item{levels}{ character vector of possible genotype/haplotype values
    stored coded by \code{paste( allele1, "/", allele2, sep="")}.}
  \item{allele.names}{ character vector of possible alleles. For a SNP,
    these might be c("A","T").   For a variable length dinucleotyde
    repeat this might be c("136","138","140","148"). }
  \item{allele.map}{ matrix encoding how the factor levels correspond to
    alleles.  See the source code to \code{allele.genotype()} for how to
    extract allele values using this matrix.  Better yet, just use
    \code{allele.genotype()}.}
  \item{genotypeOrder}{character, genotype/haplotype names in
    defined order that can used for sorting in various functions. Note
    that this slot stores both ordered and unordered genotypes
    i.e. "A/B" and "B/A".}
}

%\references{ ~put references to the literature/web site here ~ }
\author{Gregory R. Warnes \email{greg@warnes.net} and
  Friedrich Leisch.}
\seealso{
  \code{\link{HWE.test}},
  \code{\link{allele}},
  \code{\link{homozygote}},
  \code{\link{heterozygote}},
  \code{\link{carrier}},
  \code{\link{summary.genotype}},
  \code{\link{allele.count}},
  \code{\link{sort.genotype}},
  \code{\link{genotypeOrder}},
  \code{\link{locus}},
  \code{\link{gene}},
  \code{\link{marker}}, and
  \code{\link{\%in\%}} for default \%in\% method
  }

\examples{
# several examples of genotype data in different formats
example.data   <- c("D/D","D/I","D/D","I/I","D/D",
                    "D/D","D/D","D/D","I/I","")
g1  <- genotype(example.data)
g1

example.data2  <- c("C-C","C-T","C-C","T-T","C-C",
                    "C-C","C-C","C-C","T-T","")
g2  <- genotype(example.data2,sep="-")
g2


example.nosep  <- c("DD", "DI", "DD", "II", "DD",
                    "DD", "DD", "DD", "II", "")
g3  <- genotype(example.nosep,sep="")
g3

example.a1 <- c("D",  "D",  "D",  "I",  "D",  "D",  "D",  "D",  "I",  "")
example.a2 <- c("D",  "I",  "D",  "I",  "D",  "D",  "D",  "D",  "I",  "")
g4  <- genotype(example.a1,example.a2)
g4

example.mat <- cbind(a1=example.a1, a1=example.a2)
g5  <- genotype(example.mat)
g5

example.data5  <- c("D   /   D","D   /   I","D   /   D","I   /   I",
                    "D   /   D","D   /   D","D   /   D","D   /   D",
                    "I   /   I","")
g5  <- genotype(example.data5,rem=TRUE)
g5

# show how genotype and haplotype differ
data1 <- c("C/C", "C/T", "T/C")
data2 <- c("C/C", "T/C", "T/C")

test1  <- genotype( data1 )
test2  <- genotype( data2 )

test3  <-  haplotype( data1 )
test4  <-  haplotype( data2 )

test1==test2
test3==test4

test1=="C/T"
test1=="T/C"

test3=="C/T"
test3=="T/C"

## also
test1 %in% test2
test1 %in% data2
test3 %in% test4

test1 %in% "C/T"
test1 %in% "T/C"

test3 %in% "C/T"
test3 %in% "T/C"

## "Messy" example

m3  <-  c("D D/\t   D D","D\tD/   I",  "D D/   D D","I/   I",
          "D D/   D D","D D/   D D","D D/   D D","D D/   D D",
          "I/   I","/   ","/I")

genotype(m3)
summary(genotype(m3))

m4  <-  c("D D","D I","D D","I I",
          "D D","D D","D D","D D",
          "I I","   ","  I")

genotype(m4,sep=1)
genotype(m4,sep=" ",remove.spaces=FALSE)
summary(genotype(m4,sep=" ",remove.spaces=FALSE))

m5  <-  c("DD","DI","DD","II",
          "DD","DD","DD","DD",
          "II","   "," I")
genotype(m5,sep=1)
haplotype(m5,sep=1,remove.spaces=FALSE)

g5  <- genotype(m5,sep="")
h5  <- haplotype(m5,sep="")

heterozygote(g5)
homozygote(g5)
carrier(g5,"D")

g5[9:10]  <- haplotype(m4,sep=" ",remove=FALSE)[1:2]
g5

g5[9:10]
allele(g5[9:10],1)
allele(g5,1)[9:10]

# drop unused alleles
g5[9:10,drop=TRUE]
h5[9:10,drop=TRUE]

# Convert allele.counts into genotype

x <- c(0,1,2,1,1,2,NA,1,2,1,2,2,2)
g <- as.genotype.allele.count(x, alleles=c("C","T") )
g

# Use of genotypeOrder
example.data   <- c("D/D","D/I","I/D","I/I","D/D",
                    "D/D","D/I","I/D","I/I","")
summary(genotype(example.data))
genotypeOrder(genotype(example.data))

summary(genotype(example.data, genotypeOrder=c("D/D", "I/I", "D/I")))
summary(genotype(example.data, genotypeOrder=c(              "D/I")))
summary(haplotype(example.data, genotypeOrder=c(             "I/D", "D/I")))
example.data <- genotype(example.data)
genotypeOrder(example.data) <- c("D/D", "I/I", "D/I")
genotypeOrder(example.data)

}
\keyword{ misc }
%%\keyword{genetics}

% $Id: makeGenotypes.Rd 1346 2011-02-02 02:21:44Z warnes $

\name{makeGenotypes}
\alias{makeGenotypes}
\alias{makeHaplotypes}
\title{Convert columns in a dataframe to genotypes or haplotypes}
\description{ 
  Convert columns in a dataframe to genotypes or haplotypes.
  }
\usage{
makeGenotypes(data, convert, sep = "/", tol = 0.5, ..., method=as.genotype)
makeHaplotypes(data, convert, sep = "/", tol = 0.9, ...)
}
\arguments{
  \item{data}{Dataframe containing columns to be converted}
  \item{convert}{Vector or list of pairs specifying which columns
    contain genotype/haplotype data.   See below for details.}
  \item{sep}{Genotype separator}
  \item{tol}{See below.}
  \item{\dots}{Optional arguments to as.genotype function}
  \item{method}{Function used to perform the conversion.}
}
\details{
  The functions makeGenotypes and makeHaplotypes allow the conversion of
  all of the genetic variables in a dataset to genotypes or haplotypes
  in a single step.

  The parameter \code{convert} may be missing, a vector of
  column names, indexes or true/false indictators, or a list of column
  name or index pairs.

  When the argument \code{convert} is not provided, the function will
  look for columns where at least \code{tol}*100\% of the records
  contain the separator character \code{sep} ('/' by default).  These
  columns will then be assumed to contain both of the genotype/haplotype
  alleles and will be converted in-place to genotype variables.
  
  When the argument \code{convert} is a vector of column names, indexes
  or true/false indictators, the corresponding columns will be assumed
  to contain both of the genotype/haplotype alleles and will be
  converted in-place to genotype variables.

  When the argument \code{convert} is a list containing column name or
  index pairs, the two elements of each pair will be assumed to contain the
  individual alleles of a genotype/haplotype.  The first column
  specified in each pair will be replaced with the new
  genotype/haplotype variable named \code{name1 + sep + name2}.  The
  second column will be removed.

  Note that the \code{method} argument may be used to supply a
  non-standard conversion function, such as
  \code{as.genotype.allele.count}, which converts from [0,1,2] to
  ['A/A','A/B','A/C'] (or the specified allele names).  See the example
  below.
}
\value{
  Dataframe containing converted genotype/haplotype variables. All other
  variables will be unchanged.
}
\author{ Gregory R. Warnes \email{greg@warnes.net }
  }
\seealso{
  \code{\link{genotype}}
}
\examples{
\dontrun{
# common case
data <- read.csv(file="genotype_data.csv")
data <- makeGenotypes(data)
}

# Create a test data set where there are several genotypes in columns
# of the form "A/T".
test1 <- data.frame(Tmt=sample(c("Control","Trt1","Trt2"),20, replace=TRUE),
                G1=sample(c("A/T","T/T","T/A",NA),20, replace=TRUE),
                N1=rnorm(20),
                I1=sample(1:100,20,replace=TRUE),
                G2=paste(sample(c("134","138","140","142","146"),20,
                                replace=TRUE),
                         sample(c("134","138","140","142","146"),20,
                                replace=TRUE),
                         sep=" / "),
                G3=sample(c("A /T","T /T","T /A"),20, replace=TRUE),
                comment=sample(c("Possible Bad Data/Lab Error",""),20,
                               rep=TRUE)
                )
test1

# now automatically convert genotype columns
geno1 <- makeGenotypes(test1)
geno1

# Create a test data set where there are several haplotypes with alleles
# in adjacent columns.
test2 <- data.frame(Tmt=sample(c("Control","Trt1","Trt2"),20, replace=TRUE),
                    G1.1=sample(c("A","T",NA),20, replace=TRUE),
                    G1.2=sample(c("A","T",NA),20, replace=TRUE),
                    N1=rnorm(20),
                    I1=sample(1:100,20,replace=TRUE),
                    G2.1=sample(c("134","138","140","142","146"),20,
                                replace=TRUE),
                    G2.2=sample(c("134","138","140","142","146"),20,
                                replace=TRUE),
                    G3.1=sample(c("A ","T ","T "),20, replace=TRUE),
                    G3.2=sample(c("A ","T ","T "),20, replace=TRUE),
                    comment=sample(c("Possible Bad Data/Lab Error",""),20,
                                   rep=TRUE)
                   ) 
test2

# specifly the locations of the columns to be paired for haplotypes
makeHaplotypes(test2, convert=list(c("G1.1","G1.2"),6:7,8:9))

# Create a test data set where the data is coded as numeric allele
# counts (0-2).
test3 <- data.frame(Tmt=sample(c("Control","Trt1","Trt2"),20, replace=TRUE),
                    G1=sample(c(0:2,NA),20, replace=TRUE),
                    N1=rnorm(20),
                    I1=sample(1:100,20,replace=TRUE),
                    G2=sample(0:2,20, replace=TRUE),
                    comment=sample(c("Possible Bad Data/Lab Error",""),20,
                                   rep=TRUE)
                   ) 
test3

# specifly the locations of the columns, and a non-standard conversion
makeGenotypes(test3, convert=c('G1','G2'), method=as.genotype.allele.count)


}
\keyword{ misc }

% $Id: power.casectl.Rd 1272 2007-07-18 11:57:28Z ggorjan $
\name{Depreciated}
\alias{power.casectrl}
\title{Depreciated functions}
\description{
  These functions are depreciated.
}
\usage{
power.casectrl(...)
}
\arguments{
  \item{\dots}{All arguments are ignored}
}
\details{
  The \code{power.casectl} function contained serious errors.  For some
  time, replacements were provided by the BioConductor GeneticsDesign package.
      
  In specific, the \code{power.casectl} function used an expected
  contingency table to create the test statistic that was
  erroneously based on the underlying null, rather than on the
  marginal totals of the observed table. In addition, the modeling of
  dominant and recessive modes of inheritance had assumed a "perfect" 
  genotype with no disease, whereas in reality a dominant or recessive
  mode of inheritance simply means that two of the genotypes will have
  an identical odds ratio compared to the 3rd genotype (the other
  homozygote). 
}
\keyword{misc}

% $Id: ci.balance.Rd 1346 2011-02-02 02:21:44Z warnes $

\name{ci.balance}
\alias{ci.balance}
\title{Experimental Function to Correct Confidence Intervals At or Near
  Boundaries of the Parameter Space by 'Sliding' the Interval on the
  Quantile Scale.}
\description{Experimental function to correct confidence intervals at or near
  boundaries of the parameter space by 'sliding' the interval on the
  quantile scale.}
\usage{
ci.balance(x, est, confidence=0.95, alpha=1-confidence, minval, maxval,
           na.rm=TRUE)
}
\arguments{
  \item{x}{Bootstrap parameter estimates.}
  \item{est}{Observed value of the parameter.}
  \item{confidence}{Confidence level for the interval. Defaults to
    0.95.}
  \item{alpha}{Type I error rate (size) for the interval. Defaults to
               1-\code{confidence}.}  
  \item{minval}{A numeric value specifying the lower bound of the
                parameter space. Leave unspecified (the default) if
                there is no lower bound.}
  \item{maxval}{A numeric value specifying the upper bound of the
                parameter space. Leave unspecified (the default) if
                there is no upper bound.}
  \item{na.rm}{ logical. Should missing values be removed?}
}
\details{
  EXPERIMENTAL FUNCTION:

  This function attempts to compute a proper \code{conf}*100\%
  confidence interval for parameters at or near the boundary of the
  parameter space using bootstrapped parameter estimates by 'sliding'
  the confidence interval on the quantile scale.
  
  This is accomplished by attempting to place a \code{conf} *100\%
  interval symmetrically *on the quantile scale* about the observed
  value.  If a symmetric interval would exceed the observed data at the
  upper (lower) end, a one-sided interval is computed with the upper
  (lower) boundary fixed at the the upper (lower) boundary of the
  parameter space.
}
\value{
  A list containing:
  \item{ci}{A 2-element vector containing the lower and upper confidence
    limits.  The names of the elements of the vector give the actual
    quantile values used for the interval or one of the character
    strings "Upper Boundary" or "Lower Boundary".}
  \item{overflow.upper, overflow.lower}{The number of elements beyond
    those observed that would be needed to compute a symmetric (on the
    quantile scale) confidence interval.}
  \item{n.above, n.below}{The number of bootstrap values which are above
    (below) the observed value.}
  \item{lower.n, upper.n}{The index of the value used for the endpoint
    of the confidence interval or the character string "Upper Boundary"
    ("Lower Boundary").}
}
\author{ Gregory R. Warnes \email{greg@warnes.net }
}

\seealso{
  \code{\link[boot]{boot}},
  \code{\link[bootstrap]{bootstrap}},
  Used by \code{\link{diseq.ci}}.
}

\examples{
# These are nonsensical examples which simply exercise the
# computation. See the code to diseq.ci for a real example.
#
# FIXME: Add real example using boot or bootstrap.  

set.seed(7981357)
x <- abs(rnorm(100,1))
ci.balance(x,1, minval=0)
ci.balance(x,1)

x <- rnorm(100,1)
x <- ifelse(x>1, 1, x)
ci.balance(x,1, maxval=1)
ci.balance(x,1)
}
\keyword{misc}
%%\keyword{genetics}

% groupGenotype.Rd
%--------------------------------------------------------------------------
% What: Group genotype values man page
% Time-stamp: <2007-07-21 12:02:09 ggorjan>
%--------------------------------------------------------------------------

\name{groupGenotype}

\alias{groupGenotype}

\title{Group genotype values}

\description{\code{groupGenotype} groups genotype or haplotype values
  according to given "grouping/mapping" information}

\usage{

groupGenotype(x, map, haplotype=FALSE, factor=TRUE, levels=NULL, verbose=FALSE)

}

\arguments{
  \item{x}{genotype or haplotype}
  \item{map}{list, mapping information, see details and examples}
  \item{haplotype}{logical, should values in a \code{map} be treated as
    haplotypes or genotypes, see details}
  \item{factor}{logical, should output be a factor or a character}
  \item{levels}{character, optional vector of level names if factor is
    produced (\code{factor=TRUE}); the default is to use the sort order
    of the group names in \code{map}}
  \item{verbose}{logical, print genotype names that match entries in
    the map - mainly used for debugging}
}

\value{A factor or character vector with genotypes grouped}

\details{

  Examples show how \code{map} can be constructed. This are the main
  points to be aware of:
  \itemize{
    \item names of list components are used as new group names

    \item list components hold genotype names per each group

    \item genotype names can be specified directly i.e. "A/B" or
    abbreviated such as "A/*" or even "*/*", where "*" matches any
    possible allele, but read also further on

    \item all genotype names that are not specified can be captured
    with ".else" (note the dot!)

    \item genotype names that were not specified (and ".else" was not
    used) are changed to \code{NA}
  }

  \code{map} is inspected before grouping of genotypes is being
  done. The following steps are done during inspection:
  \itemize{
    \item ".else" must be at the end (if not, it is moved) to match
    everything that has not yet been defined

    \item any specifications like "A/*", "*/A", or "*/*" are extended to
    all possible genotypes based on alleles in argument \code{alleles} -
    in case of \code{haplotype=FALSE}, "A/*" and "*/A" match the same
    genotypes

    \item since use of "*" and ".else" can cause duplicates along the
    whole map, duplicates are removed sequentially (first occurrence
    is kept)
  }

  Using ".else" or "*/*" at the end of the map produces the same result,
  due to removing duplicates sequentially.
}

\author{Gregor Gorjanc}

\seealso{
  \code{\link{genotype}},
  \code{\link{haplotype}},
  \code{\link{factor}}, and
  \code{\link{levels}}

}

\examples{

## --- Setup ---

x <- c("A/A", "A/B", "B/A", "A/C", "C/A", "A/D", "D/A",
       "B/B", "B/C", "C/B", "B/D", "D/B",
       "C/C", "C/D", "D/C",
       "D/D")
g <- genotype(x, reorder="yes")
## "A/A" "A/B" "A/B" "A/C" "A/C" "A/D" "A/D" "B/B" "B/C" "B/C" "B/D" "B/D"
## "C/C" "C/D" "C/D" "D/D"

h <- haplotype(x)
## "A/A" "A/B" "B/A" "A/C" "C/A" "A/D" "D/A" "B/B" "B/C" "C/B" "B/D" "D/B"
## "C/C" "C/D" "D/C" "D/D"

## --- Use of "A/A", "A/*" and ".else" ---

map <- list("homoG"=c("A/A", "B/B", "C/C", "D/D"),
            "heteroA*"=c("A/B", "A/C", "A/D"),
            "heteroB*"=c("B/*"),
            "heteroRest"=".else")

(tmpG <- groupGenotype(x=g, map=map, factor=FALSE))
(tmpH <- groupGenotype(x=h, map=map, factor=FALSE, haplotype=TRUE))

## Show difference between genotype and haplotype treatment
cbind(as.character(h), gen=tmpG, hap=tmpH, diff=!(tmpG == tmpH))
##              gen          hap          diff
##  [1,] "A/A" "homoG"      "homoG"      "FALSE"
##  [2,] "A/B" "heteroA*"   "heteroA*"   "FALSE"
##  [3,] "B/A" "heteroA*"   "heteroB*"   "TRUE"
##  [4,] "A/C" "heteroA*"   "heteroA*"   "FALSE"
##  [5,] "C/A" "heteroA*"   "heteroRest" "TRUE"
##  [6,] "A/D" "heteroA*"   "heteroA*"   "FALSE"
##  [7,] "D/A" "heteroA*"   "heteroRest" "TRUE"
##  [8,] "B/B" "homoG"      "homoG"      "FALSE"
##  [9,] "B/C" "heteroB*"   "heteroB*"   "FALSE"
## [10,] "C/B" "heteroB*"   "heteroRest" "TRUE"
## [11,] "B/D" "heteroB*"   "heteroB*"   "FALSE"
## [12,] "D/B" "heteroB*"   "heteroRest" "TRUE"
## [13,] "C/C" "homoG"      "homoG"      "FALSE"
## [14,] "C/D" "heteroRest" "heteroRest" "FALSE"
## [15,] "D/C" "heteroRest" "heteroRest" "FALSE"
## [16,] "D/D" "homoG"      "homoG"      "FALSE"

map <- list("withA"="A/*", "rest"=".else")
groupGenotype(x=g, map=map, factor=FALSE)
##  [1] "withA" "withA" "withA" "withA" "withA" "withA" "withA" "rest"  "rest"
## [10] "rest"  "rest"  "rest"  "rest"  "rest"  "rest"  "rest"

groupGenotype(x=h, map=map, factor=FALSE, haplotype=TRUE)
##  [1] "withA" "withA" "rest"  "withA" "rest"  "withA" "rest"  "rest"  "rest"
## [10] "rest"  "rest"  "rest"  "rest"  "rest"  "rest"  "rest"

## --- Use of "*/*" ---

map <- list("withA"="A/*", withB="*/*")
groupGenotype(x=g, map=map, factor=FALSE)
##  [1] "withA" "withA" "withA" "withA" "withA" "withA" "withA" "withB" "withB"
## [10] "withB" "withB" "withB" "withB" "withB" "withB" "withB"

## --- Missing genotype specifications produces NA's ---

map <- list("withA"="A/*", withB="B/*")
groupGenotype(x=g, map=map, factor=FALSE)
##  [1] "withA" "withA" "withA" "withA" "withA" "withA" "withA" "withB" "withB"
## [10] "withB" "withB" "withB" NA      NA      NA      NA

groupGenotype(x=h, map=map, factor=FALSE, haplotype=TRUE)
##  [1] "withA" "withA" "withB" "withA" NA      "withA" NA      "withB" "withB"
## [10] NA      "withB" NA      NA      NA      NA      NA

}

\keyword{manip}
\keyword{misc}

%--------------------------------------------------------------------------
% groupGenotype.Rd ends here

% order.genotype.Rd
%--------------------------------------------------------------------------
% What: Order/sort genotype man page
% Time-stamp: <2007-07-20 03:11:39 ggorjan>
%--------------------------------------------------------------------------

\name{order.genotype}

\alias{order}
\alias{order.genotype}
\alias{sort.genotype}
\alias{genotypeOrder}
\alias{genotypeOrder<-}

\concept{order.haplotype}
\concept{sort.haplotype}

\title{Order/sort genotype/haplotype object}

\description{Order/sort genotype or haplotype object according
  to order of allele names or genotypes}

\usage{

\method{order}{genotype}(..., na.last=TRUE, decreasing=FALSE,
  alleleOrder=allele.names(x), genotypeOrder=NULL)

\method{sort}{genotype}(x, decreasing=FALSE, na.last=NA, ...,
  alleleOrder=allele.names(x), genotypeOrder=NULL)

genotypeOrder(x)
genotypeOrder(x) <- value

}

\arguments{
  \item{\ldots}{genotype or haplotype in \code{order} method; not used
    for \code{sort} method}
  \item{x}{genotype or haplotype in \code{sort} method}
  \item{na.last}{as in default \code{\link{order}} or \code{\link{sort}}}
  \item{decreasing}{as in default \code{\link{order}} or \code{\link{sort}}}
  \item{alleleOrder}{character, vector of allele names in wanted order}
  \item{genotypeOrder}{character, vector of genotype/haplotype names in
    wanted order}
  \item{value}{the same as in argument \code{order.genotype}}
}

\value{The same as in \code{order} or \code{sort}}

\details{

Argument \code{genotypeOrder} can be usefull, when you want that some
genotypes appear "together", whereas they are not "together" by allele
order.

Both methods (\code{order} and \code{sort}) work with genotype and
haplotype classes.

If \code{alleleOrder} is given, \code{genotypeOrder} has no effect.

Genotypes/haplotypes, with missing alleles in \code{alleleOrder} are
treated as \code{NA} and ordered according to \code{\link{order}}
arguments related to \code{NA} values. In such cases a warning is issued
("Found data values not matching specified alleles. Converting to NA.")
and can be safely ignored. Genotypes present in \code{x}, but not
specified in \code{genotypeOrder}, are also treated as \code{NA}.

Value of \code{genotypeOrder} such as "B/A" matches also "A/B" in case
of genotypes.

Only unique values in argument \code{alleleOrder} or
\code{genotypeOrder} are used i.e. first occurrence prevails.

}

\author{Gregor Gorjanc}

\seealso{
  \code{\link{genotype}},
  \code{\link{allele.names}},
  \code{\link{order}}, and
  \code{\link{sort}}

}

\examples{
  x <- c("C/C", "A/C", "A/A", NA, "C/B", "B/A", "B/B", "B/C", "A/C")
  alleles <- c("A", "B", "C")

  g <- genotype(x, alleles=alleles, reorder="yes")
  ## "C/C" "A/C" "A/A" NA    "B/C" "A/B" "B/B" "B/C" "A/C"

  h <- haplotype(x, alleles=alleles)
  ## "C/C" "A/C" "A/A" NA    "C/B" "B/A" "B/B" "B/C" "A/C"

  ## --- Standard usage ---

  sort(g)
  ## "A/A" "A/B" "A/C" "A/C" "B/B" "B/C" "B/C" "C/C" NA

  sort(h)
  ## "A/A" "A/C" "A/C" "B/A" "B/B" "B/C" "C/B" "C/C" NA

  ## --- Reversed order of alleles ---

  sort(g, alleleOrder=c("B", "C", "A"))
  ## "B/B" "B/C" "B/C" "A/B" "C/C" "A/C" "A/C" "A/A" NA
  ## note that A/B comes after B/C since it is treated as B/A;
  ## order of alleles (not in alleleOrder!) does not matter for a genotype

  sort(h, alleleOrder=c("B", "C", "A"))
  ## "B/B" "B/C" "B/A" "C/B" "C/C" "A/C" "A/C" "A/A" NA

  ## --- Missing allele(s) in alleleOrder ---

  sort(g, alleleOrder=c("B", "C"))
  ## "B/B" "B/C" "B/C" "C/C" "A/C" "A/A" NA    "A/B" "A/C"

  sort(g, alleleOrder=c("B"))
  ## "B/B" "C/C" "A/C" "A/A" NA    "B/C" "A/B" "B/C" "A/C"
  ## genotypes with missing allele are treated as NA

  sort(h, alleleOrder=c("B", "C"))
  ## "B/B" "B/C" "C/B" "C/C" "A/C" "A/A" NA    "B/A" "A/C"

  sort(h, alleleOrder=c("B"))
  ## "B/B" "C/C" "A/C" "A/A" NA    "C/B" "B/A" "B/C" "A/C"

  ## --- Use of genotypeOrder ---

  sort(g, genotypeOrder=c("A/A", "C/C", "B/B", "A/B", "A/C", "B/C"))
  ## "A/A" "C/C" "B/B" "A/B" "A/C" "A/C" "B/C" "B/C" NA

  sort(h, genotypeOrder=c("A/A", "C/C", "B/B",
                          "A/C", "C/B", "B/A", "B/C"))
  ## "A/A" "C/C" "B/B" "A/C" "A/C" "C/B" "B/A" "B/C" NA

  ## --- Missing genotype(s) in genotypeOrder ---

  sort(g, genotypeOrder=c(       "C/C",        "A/B", "A/C", "B/C"))
  ## "C/C" "A/B" "A/C" "A/C" "B/C" "B/C" "A/A" NA    "B/B"

  sort(h, genotypeOrder=c(       "C/C",        "A/B", "A/C", "B/C"))
  ## "C/C" "A/C" "A/C" "B/C" "A/A" NA    "C/B" "B/A" "B/B"
}

\keyword{manip}
\keyword{misc}

%--------------------------------------------------------------------------
% order.genotype.Rd ends here

% $Id: HWE.chisq.Rd 1344 2011-01-17 19:13:09Z warnes $

\name{HWE.chisq}
\alias{HWE.chisq}
\alias{HWE.chisq.genotype}

\title{Perform Chi-Square Test for Hardy-Weinberg Equilibrium}

\description{
  Test the null hypothesis that Hardy-Weinberg equilibrium holds using
  the Chi-Square method.
}

\usage{
HWE.chisq(x, ...)
\method{HWE.chisq}{genotype}(x, simulate.p.value=TRUE, B=10000, ...)
          
}
\arguments{
  \item{x}{genotype or haplotype object.}
  \item{simulate.p.value}{a logical value indicating whether the p-value
    should be computed using simulation instead of using the
    \eqn{\chi^2}{Chi-Square} approximation. Defaults to \code{TRUE}.}
  \item{B}{Number of simulation iterations to use when
    \code{simulate.p.value=TRUE}. Defaults to 10000.}
  \item{...}{ optional parameters passed to \code{chisq.test}}
}
\details{
  This function generates a 2-way table of allele counts, then calls
  \code{\link{chisq.test}} to compute a p-value for Hardy-Weinberg
  Equilibrium.  By default, it uses an unadjusted Chi-Square test
  statistic and computes the p-value using a simulation/permutation
  method.  When \code{simulate.p.value=FALSE}, it computes the test
  statistic using the Yates continuity correction and tests it against
  the asymptotic Chi-Square distribution with the approproate degrees of
  freedom.

  Note: The Yates continuty correction is applied *only* when
  \code{simulate.p.value=FALSE}, so that the reported test statistics
  when \code{simulate.p.value=FALSE} and \code{simulate.p.value=TRUE}
  will differ.
}
\value{
  An object of class \code{htest}.
}

\seealso{
  \code{\link{HWE.exact}},
  \code{\link{HWE.test}},
  \code{\link{diseq}},
  \code{\link{diseq.ci}},
  \code{\link{allele}},
  \code{\link{chisq.test}},
  \code{\link[boot]{boot}},
  \code{\link[boot]{boot.ci}}
}


\examples{
\testonly{
set.seed(4657613)
}
example.data   <- c("D/D","D/I","D/D","I/I","D/D",
                    "D/D","D/D","D/D","I/I","")
g1  <- genotype(example.data)
g1

HWE.chisq(g1)
# compare with
HWE.exact(g1)
# and 
HWE.test(g1)

three.data   <- c(rep("A/A",8),
                  rep("C/A",20),
                  rep("C/T",20),
                  rep("C/C",10),
                  rep("T/T",3))

g3  <- genotype(three.data)
g3

HWE.chisq(g3, B=10000)


}
\keyword{ misc }
%%\keyword{genetics}

% $Id: write.pop.file.Rd 1346 2011-02-02 02:21:44Z warnes $

\name{write.pop.file}
\alias{write.pop.file}
\alias{write.pedigree.file}
\alias{write.marker.file}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{Create genetics data files}
\description{
  \code{write.pop.file} creates a 'pop' data file, as used by the
  GenePop (\url{https://genepop.curtin.edu.au/}) and LinkDos
  (\url{https://genepop.curtin.edu.au/linkC.html}) software
  packages.

  \code{write.pedigree.file} creates a 'pedigree' data file, as used
  by the QTDT software package
  (\url{http://csg.sph.umich.edu//abecasis/QTDT/}).
  
  \code{write.marker.file} creates a 'marker' data file, as used by
  the QTDT software package
  (\url{http://csg.sph.umich.edu//abecasis/QTDT/}).
}
\usage{
write.pop.file(data, file = "", digits = 2, description = "Data from R")
write.pedigree.file(data, family, pid, father, mother, sex,
                    file="pedigree.txt")
write.marker.file(data, location, file="marker.txt")
}
\arguments{
  \item{data}{Data frame containing genotype objects to be exported}
  \item{file}{Output filename}
  \item{digits}{Number of digits to use in numbering genotypes, either 2
    or 3.}
  \item{description}{Description to use as the first line of the 'pop'
    file.}
  \item{family, pid, father, mother}{Vector of family, individual,
    father, and mother id's, respectively.}
  \item{sex}{Vector giving the sex of the individual (1=Make, 2=Female)}
  \item{location}{Location of the marker relative to the gene of
    interest, in base pairs.}
}
\details{
  The format of 'Pop' files is documented at
  \url{https://genepop.curtin.edu.au/help_input.html}, the format
  of 'pedigree' files is documented at \url{http://csg.sph.umich.edu/abecasis/GOLD/docs/pedigree.html} and the format of 'marker'
  files is documented at
  \url{http://csg.sph.umich.edu/abecasis/GOLD/docs/map.html}.
}
\value{
  No return value.
}
\author{Gregory R. Warnes \email{greg@warnes.net}}
\seealso{\code{\link{write.table}}}
\examples{
  # TBA
}
\keyword{IO}

% $Id: print.LD.Rd 1353 2012-08-14 14:23:17Z warnes $

\name{print.LD}
\alias{print.LD}
\alias{print.LD.data.frame}
\alias{summary.LD.data.frame}
\alias{print.summary.LD.data.frame}
\alias{plot.LD.data.frame}
\alias{LDtable}
\alias{LDplot}
\title{Textual and graphical display of linkage disequilibrium (LD) objects}
\description{
  Textual and graphical display of linkage disequilibrium (LD) objects
}
\usage{

\method{print}{LD}(x, digits = getOption("digits"), \dots)
\method{print}{LD.data.frame}(x, \dots)

\method{summary.LD}{data.frame}(object, digits = getOption("digits"),
                      which = c("D", "D'", "r", "X^2", "P-value", "n", " "),
                      rowsep, show.all = FALSE, \dots)
\method{print}{summary.LD.data.frame}(x, digits = getOption("digits"), \dots)

\method{plot}{LD.data.frame}(x,digits=3, colorcut=c(0,0.01, 0.025, 0.5, 0.1, 1),
                   colors=heat.colors(length(colorcut)), textcol="black",
                   marker, which="D'", distance,  \dots)


LDtable(x, colorcut=c(0,0.01, 0.025, 0.5, 0.1, 1),
        colors=heat.colors(length(colorcut)), textcol="black",
        digits=3, show.all=FALSE, which=c("D", "D'", "r", "X^2",
        "P-value", "n"), colorize="P-value", cex, \dots)

LDplot(x, digits=3, marker, distance, which=c("D", "D'", "r", "X^2",
       "P-value", "n", " "), \dots )
}
\arguments{
  \item{x,object}{LD or LD.data.frame object}
  \item{digits}{Number of significant digits to display}
  \item{which}{Name(s) of LD information items to be displayed}
  \item{rowsep}{Separator between rows of data, use \code{NULL} for no
    separator.}
  \item{colorcut}{P-value cutoffs points for colorizing LDtable}
  \item{colors}{Colors for each P-value cutoff given in \code{colorcut} for
    LDtable}
  \item{textcol}{Color for text labels for LDtable}
  \item{marker}{Marker used as 'comparator' on LDplot.  If
    omitted separate lines for each marker will be displayed}
  \item{distance}{Marker location, used for locating of markers on
    LDplot.}
  \item{show.all}{If TRUE, show all rows/columns of matrix. Otherwise
    omit completely blank rows/columns.}
  \item{colorize}{LD parameter used for determining table cell colors}
  \item{cex}{Scaling factor for table text. If absent, text will be
    scaled to fit within the table cells.}
  \item{\dots}{Optional arguments (\code{plot.LD.data.frame} passes
    these to \code{LDtable} and \code{LDplot})}
}
%\details{
%}
\value{
  None.
}
%\references{ ~put references to the literature/web site here ~ }
\author{ Gregory R. Warnes \email{greg@warnes.net} }
\seealso{ \code{LD}, \code{genotype}, \code{HWE.test} }
\examples{


g1 <- genotype( c('T/A',    NA, 'T/T',    NA, 'T/A',    NA, 'T/T', 'T/A',
                  'T/T', 'T/T', 'T/A', 'A/A', 'T/T', 'T/A', 'T/A', 'T/T',
                     NA, 'T/A', 'T/A',   NA) )

g2 <- genotype( c('C/A', 'C/A', 'C/C', 'C/A', 'C/C', 'C/A', 'C/A', 'C/A',
                  'C/A', 'C/C', 'C/A', 'A/A', 'C/A', 'A/A', 'C/A', 'C/C',
                  'C/A', 'C/A', 'C/A', 'A/A') )


g3 <- genotype( c('T/A', 'T/A', 'T/T', 'T/A', 'T/T', 'T/A', 'T/A', 'T/A',
                  'T/A', 'T/T', 'T/A', 'T/T', 'T/A', 'T/A', 'T/A', 'T/T',
                  'T/A', 'T/A', 'T/A', 'T/T') )
data <- makeGenotypes(data.frame(g1,g2,g3))

# Compute & display  LD for one marker pair
ld <- LD(g1,g2)
print(ld)

# Compute LD table for all 3 genotypes
ldt <- LD(data)

# display the results
print(ldt)                               # textual display
LDtable(ldt)                            # graphical color-coded table
LDplot(ldt, distance=c(124, 834, 927))  # LD plot vs distance

# more markers makes prettier plots!
data <- list()
nobs <- 1000
ngene <- 20
s <- seq(0,1,length=ngene)
a1 <- a2 <- matrix("", nrow=nobs, ncol=ngene)
for(i in 1:length(s) )
{

  rallele <- function(p) sample( c("A","T"), 1, p=c(p, 1-p))

  if(i==1)
    {
      a1[,i] <- sample( c("A","T"), 1000, p=c(0.5,0.5), replace=TRUE)
      a2[,i] <- sample( c("A","T"), 1000, p=c(0.5,0.5), replace=TRUE)
    }
  else
    {
      p1 <- pmax( pmin( 0.25 + s[i] * as.numeric(a1[,i-1]=="A"),1 ), 0 )
      p2 <- pmax( pmin( 0.25 + s[i] * as.numeric(a2[,i-1]=="A"),1 ), 0 )
      a1[,i] <- sapply(p1, rallele )
      a2[,i] <- sapply(p2, rallele )
    }

  data[[paste("G",i,sep="")]] <- genotype(a1[,i],a2[,i])
}
data <- data.frame(data)
data <- makeGenotypes(data)

ldt <- LD(data)
plot(ldt, digits=2, marker=19) # do LDtable & LDplot on in a single
                               # graphics window
}
\keyword{misc}

\name{plot.genotype}

\alias{plot.genotype}

\title{Plot genotype object}

\description{
  \code{plot.genotype} can plot genotype or allele frequency of a genotype
  object.
}

\usage{
\method{plot}{genotype}(x, type=c("genotype", "allele"),
  what=c("percentage", "number"), ...)
}

\arguments{
  \item{x}{genotype object, as genotype.}
  \item{type}{plot "genotype" or "allele" frequency, as character.}
  \item{what}{show "percentage" or "number", as character}
  \item{\dots}{Optional arguments for \code{barplot}.}
}

\value{
  The same as in \code{barplot}.
}

\author{ Gregor Gorjanc }

\seealso{ \code{\link{genotype}}, \code{\link{barplot}} }

\examples{
  set <- c("A/A", "A/B", "A/B", "B/B", "B/B", "B/B",
           "B/B", "B/C", "C/C", "C/C")
  set <- genotype(set, alleles=c("A", "B", "C"), reorder="yes")
  plot(set)
  plot(set, type="allele", what="number")
}

\keyword{hplot}

% $Id: LD.Rd 1346 2011-02-02 02:21:44Z warnes $

\name{LD}
\alias{LD}
\alias{LD.genotype}
\alias{LD.data.frame}
\title{Pairwise linkage disequilibrium between genetic markers.}
\description{
  Compute pairwise linkage disequilibrium between genetic markers
}
\usage{
LD(g1, ...)
\method{LD}{genotype}(g1,g2,...)
\method{LD}{data.frame}(g1,...)
}
\arguments{
  \item{g1}{ genotype object or dataframe containing genotype objects }
  \item{g2}{ genotype object (ignored if g1 is a dataframe) }
  \item{\dots}{ optional arguments (ignored) }
}
\details{
  Linkage disequilibrium (LD) is the non-random association of
  marker alleles and can arise from marker proximity or from selection
  bias.
  
  \code{LD.genotype} estimates the extent of LD for a single pair of
  genotypes.  \code{LD.data.frame} computes LD for all pairs of
  genotypes contained in a data frame.  Before starting,
  \code{LD.data.frame} checks the class and number of alleles of each
  variable in the dataframe.  If the data frame contains non-genotype
  objects or genotypes with more or less than 2 alleles, these will be
  omitted from the computation and a warning will be generated.

  Three estimators of LD are computed:
  \itemize{

    \item{D}{ raw difference in frequency between the
              observed number of AB pairs and the expected number:
      
      \deqn{%
 	D = p_{AB} - p_A p_B %
      }{%
 	D = p(AB) - p(A)*p(B) %
      }
      
    }
    \item{D'}{ scaled D spanning the range [-1,1] 
      
      
      \deqn{D' = \frac{D}{D_{max} } }{D' = D / Dmax}
      
      where, if D > 0:
      \deqn{%
  	D_{max} = \min( p_A p_b, p_a p_B )  %
      }{%
  	Dmax = min( p(A)p(b), p(a)p(B) )   %
      } 
      or if D < 0:
      \deqn{%
  	D_{max} = \max{ -p_A p_B, -p_a p_b }  %
      }{%
  	Dmax = max( -p(A)p(B), -p(a)p(b) )  %
      }
    }
    
    \item{r}{ correlation coefficient between the markers
      
      \deqn{%
 	r = \frac{-D}{\sqrt( p_A * p_a * p_B * p_b  )} %
      }{%
 	r = -D / sqrt( p(A) * p(a) * p(B) * p(b) ) %
      }
    }
  }
  
  where
  \itemize{
    \item{-}{ \eqn{p_A}{p(A)} is defined as the observed probability of
      allele 'A' for marker 1, }
    \item{-}{ \eqn{p_a=1-p_A}{p(a) = 1-p(A)} is defined as the observed probability of
      allele 'a' for marker 1, }
    \item{-}{\eqn{p_B}{p(B)} is defined as the observed probability of
      allele 'B' for marker 2, and }
    \item{-}{\eqn{p_b=1-p_B}{p(b) = 1- p(B)} is defined as the observed probability of
      allele 'b' for marker 2, and }
    \item{-}{\eqn{p_{AB}}{p(AB)} is defined as the probability of
      the marker allele pair 'AB'. }
  }

  For genotype data, AB/ab cannot be distinguished from
  aB/Ab. Consequently, we estimate \eqn{p_{AB}}{p(AB)} using maximum
  likelihood and use this value in the computations.
  }
\value{

  \code{LD.genotype} returns a 5 element list:
    \item{call}{the matched call}
    \item{D}{Linkage disequilibrium estimate}
    \item{Dprime }{Scaled linkage disequilibrium estimate}
    \item{corr}{Correlation coefficient} 
    \item{nobs}{Number of observations}
    \item{chisq}{Chi-square statistic for linkage
      equilibrium (i.e., D=D'=corr=0)}
    \item{p.value}{Chi-square p-value for marker independence}

  \code{LD.data.frame} returns a list with the same elements, but each
  element is a matrix where the upper off-diagonal elements contain the
  estimate for the corresponding pair of markers.  The other matrix
  elements are \code{NA}.
}
%\references{ ~put references to the literature/web site here ~ }
\author{ Gregory R. Warnes \email{greg@warnes.net} }
\seealso{ \code{\link{genotype}}, \code{\link{HWE.test}}  }
\examples{

g1 <- genotype( c('T/A',    NA, 'T/T',    NA, 'T/A',    NA, 'T/T', 'T/A',
                  'T/T', 'T/T', 'T/A', 'A/A', 'T/T', 'T/A', 'T/A', 'T/T',
                     NA, 'T/A', 'T/A',   NA) )

g2 <- genotype( c('C/A', 'C/A', 'C/C', 'C/A', 'C/C', 'C/A', 'C/A', 'C/A',
                  'C/A', 'C/C', 'C/A', 'A/A', 'C/A', 'A/A', 'C/A', 'C/C',
                  'C/A', 'C/A', 'C/A', 'A/A') )


g3 <- genotype( c('T/A', 'T/A', 'T/T', 'T/A', 'T/T', 'T/A', 'T/A', 'T/A',
                  'T/A', 'T/T', 'T/A', 'T/T', 'T/A', 'T/A', 'T/A', 'T/T',
                  'T/A', 'T/A', 'T/A', 'T/T') )

# Compute LD on a single pair

LD(g1,g2)

# Compute LD table for all 3 genotypes

data <- makeGenotypes(data.frame(g1,g2,g3))
LD(data)
}
\keyword{misc}

% $Id: undocumented.Rd 1353 2012-08-14 14:23:17Z warnes $

\name{undocumented}
\alias{as.factor}
\alias{allele.count.2.genotype}
\alias{as.factor.allele.genotype}
\alias{as.factor.default}
\alias{as.factor.genotype}
\alias{shortsummary.genotype}
\alias{geno.as.array}
\alias{mknum}
\alias{hap}
\alias{hapshuffle}
\alias{hapenum}
\alias{hapfreq}
\alias{hapmcmc}
\alias{mourant}
\alias{hapambig}
\title{Undocumented functions}
\description{
  These functions are undocumented.  Some are internal and not intended
  for direct use.  Some are not yet ready for end users.  Others simply
  haven't been documented yet.
}
\author{Gregory R. Warnes}
\keyword{ misc }
%%\keyword{genetics}

% $Id: diseq.Rd 1346 2011-02-02 02:21:44Z warnes $

\name{diseq}
\alias{diseq}
\alias{diseq.table}
\alias{diseq.genotype}
\alias{diseq.ci}
\alias{print.diseq}
\title{Estimate or Compute Confidence Interval for the Single-Marker Disequilibrium}
\description{
  Estimate or compute confidence interval for single-marker disequilibrium.
}
\usage{
diseq(x, ...)
\method{print}{diseq}(x, show=c("D","D'","r","R^2","table"), ...)
diseq.ci(x, R=1000, conf=0.95, correct=TRUE, na.rm=TRUE, ...)
}
\arguments{
  \item{x}{genotype or haplotype object.}
  \item{show}{a character value or vector indicating which
    disequilibrium measures should be displayed.  The default is to show
    all of the available measures. \code{show="table"} will display
    a table of observed, expected, and observed-expected
    frequencies.}
  \item{conf}{Confidence level to use when computing the confidence
    level for D-hat.  Defaults to 0.95, should be in (0,1). }
  \item{R}{Number of bootstrap iterations to use when computing the
    confidence interval. Defaults to 1000.}
  \item{correct}{See details.}
  \item{na.rm}{ logical. Should missing values be removed?}
  \item{...}{ optional parameters passed to \code{boot.ci}
    (\code{diseq.ci}) or ignored.}
}
\details{
  For a single-gene marker, \code{diseq} computes the Hardy-Weinberg
  (dis)equilibrium statistic D, D', r (the correlation coefficient), and
  \eqn{r^2} for each pair of allele values, as well as an overall
  summary value for each measure across all alleles.  \code{print.diseq}
  displays the contents of a \code{diseq} object. \code{diseq.ci}
  computes a bootstrap confidence interval for this estimate.

  For consistency, I have applied the standard definitions for D, D',
  and r from the Linkage Disequilibrium case, replacing all marker 
  probabilities with the appropriate allele probabilities.
  
  Thus, for each allele pair,
  \itemize{
    \item{D}{ is defined as the half of the raw difference
      in frequency between
      the observed number of heterozygotes and the expected number:
      
      \deqn{%
 	D = \frac{1}{2} ( p_{ij} + p_{ji} ) - p_i p_j  %
      }{%
 	D = 1/2 * ( p(ij) + p(ji) ) - p(i)*p(j)   %
      }
      
    }
    \item{D'}{ rescales D to span the range [-1,1] 
      
      
      \deqn{D' = \frac{D}{D_{max} } }{D' = D / Dmax}
      
      where, if D > 0:
      \deqn{%
  	D_{max} = \min{ p_i p_j, p_j p_i } = p_i p_j   %
      }{%
  	Dmax = min(p(i)p(j), p(j)p(i)) =  p(i)p(j)   %
      } 
      or if D < 0:
      \deqn{%
  	D_{max} = \min{ p_i (1 - p_j), p_j (1 - p_i) } %
      }{%
  	Dmax = min( p(i) * (1 - p(j)), p(j)( 1 - (1-p(i) ) ) )
      }
    }
    
    \item{r}{ is the correlation coefficient between two alleles,
      %ignoring all other alleles,
      and can be computed by
      
      \deqn{%
 	r = \frac{-D}{\sqrt( p_i * (1-p_i) p(j) (1-p_j ) )} %
      }{%
 	r = -D / sqrt( p(i)*(1-p(i)) * p(j)*(1-p(j)) ) %
      }
    }
    
  }
  
  where
  \itemize{
    \item{-}{ \eqn{p_i}{p(i)} defined as the observed probability of
      allele 'i', }
    \item{-}{\eqn{p_j}{p(j)} defined as the observed probability of
      allele 'j', and }
    \item{-}{\eqn{p_{ij}}{p(ij)} defined as the observed probability of
      the allele pair 'ij'. }
  }
  
  When there are more than two alleles, the summary values for these
  statistics are obtained by computing a weighted average of the
  absolute value of each allele pair, where the weight is determined by
  the expected frequency. For example:

   \deqn{%
     D_{overall} = \sum_{i \ne j}  |D_{ij}| * p_{ij} % 
   }{%
     D.overall = sum |D(ij)| * p(ij) %
   }

  Bootstrapping is used to generate confidence interval in order to
  avoid reliance on parametric assumptions, which will not hold for
  alleles with low frequencies (e.g. \eqn{D'} following a a Chi-square 
  distribution).  

  See the function \code{\link[genetics]{HWE.test}} for testing
  Hardy-Weinberg Equilibrium, \eqn{D=0}.
  
}
\value{
  \code{diseq} returns an object of class \code{diseq} with components
  \itemize{
    \item{call}{function call used to create this object}
    \item{data}{2-way table of allele pair counts}
    \item{D.hat}{matrix giving the observed count, expected count,
      observed - expected difference, and estimate of disequilibrium for
      each pair of alleles as well as an overall disequilibrium value.}
    \item{TODO}{more slots to be documented}
  }

  \code{diseq.ci} returns an object of class \code{\link[boot]{boot.ci}}
}
\author{ Gregory R. Warnes \email{greg@warnes.net }
  }

\seealso{
  \code{\link{genotype}},
  \code{\link{HWE.test}},
  \code{\link[boot]{boot}},
  \code{\link[boot]{boot.ci}}
}

\examples{
\testonly{
set.seed(7981357)
}
example.data   <- c("D/D","D/I","D/D","I/I","D/D",
                    "D/D","D/D","D/D","I/I","")
g1  <- genotype(example.data)
g1

diseq(g1)
diseq.ci(g1)
HWE.test(g1)  # does the same, plus tests D-hat=0

three.data   <- c(rep("A/A",8),
                  rep("C/A",20),
                  rep("C/T",20),
                  rep("C/C",10),
                  rep("T/T",3))

g3  <- genotype(three.data)
g3

diseq(g3)
diseq.ci(g3, ci.B=10000, ci.type="bca")

# only show observed vs expected table
print(diseq(g3),show='table')

}
\keyword{misc}
%%\keyword{genetics}

% $Id: HWE.exact.Rd 1346 2011-02-02 02:21:44Z warnes $

\name{HWE.exact}
\alias{HWE.exact}
\title{Exact Test of Hardy-Weinberg Equilibrium for 2-Allele Markers}
\description{
  Exact test of Hardy-Weinberg Equilibrium for 2 Allele Markers.
}
\usage{
HWE.exact(x)
}
\arguments{
  \item{x}{ Genotype object }
}
%\details{
%}
\value{
  Object of class 'htest'.
}
\references{
    Emigh TH. (1980) "Comparison of tests for Hardy-Weinberg Equilibrium",
    Biometrics, 36, 627-642.
  }
\author{
  David Duffy \email{davidD@qimr.edu.au} with modifications by Gregory
  R. Warnes \email{greg@warnes.net}
  }
\note{ This function only works for genotypes with exactly 2 alleles.}
\seealso{
  \code{\link{HWE.chisq}},
  \code{\link{HWE.test}},
  \code{\link{diseq}},
  \code{\link{diseq.ci}}
}

\examples{
example.data   <- c("D/D","D/I","D/D","I/I","D/D",
                    "D/D","D/D","D/D","I/I","")
g1  <- genotype(example.data)
g1

HWE.exact(g1)
# compare with
HWE.chisq(g1)


\testonly{
set.seed(465764)
}

g2 <- genotype(sample( c("A","C"), 100, p=c(100,10), rep=TRUE),
               sample( c("A","C"), 100, p=c(100,10), rep=TRUE) )
HWE.exact(g2)

}
\keyword{ misc }
%%\keyword{genetics}

% $Id: locus.Rd 1346 2011-02-02 02:21:44Z warnes $

\name{locus}
\alias{locus}
\alias{gene}
\alias{marker}
\alias{is.gene}
\alias{is.locus}
\alias{is.marker}
\alias{print.gene}
\alias{print.locus}
\alias{print.marker}
\alias{as.character.locus}
\alias{as.character.gene}
\alias{as.character.marker}
\alias{getlocus}
\alias{getmarker}
\alias{getgene}
\alias{locus<-}
\alias{marker<-}
\alias{gene<-}


%- Also NEED an `\alias' for EACH other topic documented here.
\title{ Create and Manipulate Locus, Gene, and Marker Objects}
\description{
  \code{locus}, \code{gene}, and \code{marker} create objects to store
  information, respectively, about genetic loci, genes, and markers.

  \code{is.locus}, \code{is.gene}, and \code{ismarker} test whether an
  object is a member of the respective class.

  \code{as.character.locus}, \code{as.character.gene},
  \code{as.character.marker} return a character string containing a
  compact encoding the object.

  \code{getlocus}, \code{getgene}, \code{getmarker} extract locus data
  (if present) from another object.

  \code{locus<-}, \code{marker<-}, and \code{gene<-} adds locus data to
  an object.

}
\usage{
  locus(name, chromosome, arm=c("p", "q", "long", "short", NA),
        index.start, index.end=NULL)

  gene(name, chromosome, arm=c("p", "q", "long", "short"),
       index.start, index.end=NULL)

  marker(name, type, locus.name, bp.start, bp.end = NULL,
         relative.to = NULL, ...)

  is.locus(x)

  is.gene(x)

  is.marker(x)

  \method{as.character}{locus}(x, ...)

  \method{as.character}{gene}(x, ...)

  \method{as.character}{marker}(x, ...)

  getlocus(x, ...)

  locus(x) <- value

  marker(x) <- value

  gene(x) <- value

}
\arguments{
  \item{name}{character string giving locus, gene, or marker name}
  \item{chromosome}{integer specifying chromosome number (1:23 for humans).}
  \item{arm}{character indicating long or short arm of the chromosome.
    Long is be specified by "long" or "p".  Short is specified by
    "short" or "q".}
  \item{index.start}{integer specifying location of start of locus or
    gene on the chromosome.  }
  \item{index.end}{optional integer specifying location of end of locus or
    gene on the chromosome.  }
  \item{type}{character string indicating marker type, e.g. "SNP"}
  \item{locus.name}{either a character string giving the name of the
    locus or gene (other details may be specified using \code{...}) or a
    \code{locus} or \code{gene} object.}
  \item{bp.start}{start location of marker, in base pairs}
  \item{bp.end}{end location of marker, in base pairs (optional)}
  \item{relative.to}{location (optional) from which \code{bp.start} and
    \code{bp.end} are calculated. }
  \item{...}{parameters for \code{locus} used to fill in additional
    details on the locus or gene within which the marker is located. }
  \item{x}{an object of class \code{locus}, \code{gene}, or
    \code{marker}, or (for \code{getlocus}, \code{locus<-},
    \code{marker<-}, and \code{gene<-}) an object that may contain a locus
    attribute or field, notably a \code{genotype} object.}
  \item{value}{\code{locus}, \code{marker}, or \code{gene} object}
}
%\details{
%  ~~ If necessary, more details than the __description__  above ~~
%}
\value{
  Object of class \code{locus} and \code{gene}are lists with the
  elements:
  \item{name}{character string giving locus, gene, or marker name}
  \item{chromosome}{integer specifying chromosome number (1:23 for humans).}
  \item{arm}{character indicating long or short arm of the chromosome.
    Long is be specified by "long" or "p".  Short is specified by
    "short" or "q".}
  \item{index.start}{integer specifying location of start of locus or
    gene on the chromosome.  }
  \item{index.end}{optional integer specifying location of end of locus or
    gene on the chromosome.  }

  Objects of class \code{marker} add the additional fields:
  \item{marker.name}{character string giving the name of the marker}
  \item{bp.start}{start location of marker, in base pairs}
  \item{bp.end}{end location of marker, in base pairs (optional)}
  \item{relative.to}{location (optional) from which \code{bp.start} and
    \code{bp.end} are calculated. }

}
%\references{ ~put references to the literature/web site here ~ }
\author{Gregory R. Warnes \email{greg@warnes.net} }

\seealso{
  \code{\link{genotype}},
}

\examples{
ar2  <- gene("AR2",chromosome=7,arm="q",index.start=35)
ar2

par  <- locus(name="AR2 Psedogene",
              chromosome=1,
              arm="q",
              index.start=32,
              index.end=42)
par

c109t  <- marker(name="C-109T",
                 type="SNP",
                 locus.name="AR2",
                 chromosome=7,
                 arm="q",
                 index.start=35,
                 bp.start=-109,
                 relative.to="start of coding region")
c109t

c109t  <- marker(name="C-109T",
                 type="SNP",
                 locus=ar2,
                 bp.start=-109,
                 relative.to="start of coding region")
c109t




example.data   <- c("D/D","D/I","D/D","I/I","D/D",
                    "D/D","D/D","D/D","I/I","")
g1  <- genotype(example.data, locus=ar2)
g1

getlocus(g1)

summary(g1)
HWE.test(g1)

g2  <- genotype(example.data, locus=c109t)
summary(g2)

getlocus(g2)

heterozygote(g2)
homozygote(g1)

allele(g1,1)

carrier(g1,"I")

heterozygote(g2)
}
\keyword{ misc }
%%\keyword{genetics}%-- one or more ...

% $Id: summary.genotype.Rd 1346 2011-02-02 02:21:44Z warnes $

\name{summary.genotype}

\alias{summary.genotype}
\alias{print.summary.genotype}

\title{ Allele and Genotype Frequency from a Genotype or
  Haplotype Object}

\description{
    \code{summary.genotype} creates an object containing allele and
    genotype frequency from a \code{genotype} or \code{haplotype}
    object.  \code{print.summary.genotype} displays a
    \code{summary.genotype} object.
}

\usage{
  \method{summary}{genotype}(object, ..., maxsum)
  \method{print}{summary.genotype}(x,...,round=2)
}

\arguments{

  \item{object, x}{ an object of class \code{genotype} or \code{haplotype} (for
    \code{summary.genotype}) or an object of class
    \code{summary.genotype} (for \code{print.summary.genotype}) }
  \item{\dots}{ optional parameters.  Ignored by \code{summary.genotype},
    passed to \code{print.matrix} by \code{print.summary,genotype}.}
  \item{maxsum}{ specifying any value for the parameter
    maxsum will cause \code{summary.genotype} to fall back to
    \code{summary.factor}.}
  \item{round}{ number of digits to use when displaying proportions.}
 }

\details{
 Specifying any value for the parameter \code{maxsum} will cause fallback
 to \code{summary.factor}.  This is so that the function
 \code{summary.dataframe} will give reasonable output when it contains a
 genotype column.  (Hopefully we can figure out something better to do
 in this case.)
}


\value{
  The returned value of \code{summary.genotype} is an object of class
  \code{summary.genotype} which
  is a list with the following components:

  \item{locus }{locus information field (if present) from \code{x}}.

  \item{allele.names}{ vector of allele names }

  \item{allele.freq }{
    A two column matrix with one row for each allele, plus one row for
    \code{NA} values (if present).  The first column, \code{Count},
    contains the frequency of the corresponding allele value.  The
    second column, \code{Proportion}, contains the fraction of alleles
    with the corresponding allele value.  Note each observation contains
    two alleles, thus the \code{Count} field sums to twice the number of
    observations.
  }

  \item{genotype.freq}{
    A two column matrix with one row for each genotype, plus one row for
    \code{NA} values (if present). The first column, \code{Count}, contains the
    frequency of the corresponding genotype.  The second column,
    \code{Proportion}, contains the fraction of genotypes with the
    corresponding value.
  }

  \code{print.summary.genotype} silently returns the object \code{x}.
}
%\references{ ~put references to the literature/web site here ~ }
\author{ Gregory R. Warnes \email{greg@warnes.net} }
%\note{ ~~further notes~~ }
\seealso{
  \code{\link{genotype}},
  \code{\link{HWE.test}},
  \code{\link{allele}},
  \code{\link{homozygote}},
  \code{\link{heterozygote}},
  \code{\link{carrier}},
%  \code{\link{summary.genotype}},
  \code{\link{allele.count}}
  \code{\link{locus}}
  \code{\link{gene}}
  \code{\link{marker}}
  }

\examples{

example.data   <- c("D/D","D/I","D/D","I/I","D/D",
                    "D/D","D/D","D/D","I/I","")
g1  <- genotype(example.data)
g1

summary(g1)
}
\keyword{ misc }
%%\keyword{genetics}

% : HWE.test.Rd,v 1.13 2004/05/25 19:40:02 warnesgr Exp $

\name{HWE.test}
\alias{HWE.test}
\alias{HWE.test.genotype}
\alias{HWE.test.data.frame}
\alias{print.HWE.test}

\title{Estimate Disequilibrium and Test for Hardy-Weinberg Equilibrium}

\description{
  Estimate disequilibrium parameter and test the null hypothesis that 
  Hardy-Weinberg equilibrium holds. 
}

\usage{
HWE.test(x, ...)
\method{HWE.test}{genotype}(x, exact = nallele(x)==2, simulate.p.value=!exact,
         B=10000, conf=0.95, ci.B=1000, ... )
\method{HWE.test}{data.frame}(x, ..., do.Allele.Freq=TRUE, do.HWE.test=TRUE)
\method{print}{HWE.test}(x, show=c("D","D'","r","table"), ...)
}
\arguments{
  \item{x}{genotype or haplotype object.}
  \item{exact}{a logical value indicated whether the p-value should be
    computed using the exact method, which is only available for 2
    allele genotypes.}
  \item{simulate.p.value}{a logical value indicating whether the p-value
    should be computed using simulation instead of using the
    \eqn{\chi^2}{Chi-Square} approximation. Defaults to \code{TRUE}.}
  \item{B}{Number of simulation iterations to use when
    \code{simulate.p.value=TRUE}. Defaults to 10000.}
  \item{conf}{Confidence level to use when computing the confidence
    level for D-hat.  Defaults to 0.95, should be in (0,1). }
  \item{ci.B}{Number of bootstrap iterations to use when computing the
    confidence interval. Defaults to 1000.}
%   \item{ci.type}{Method of calculating the confidence interval using the
%     bootstrap sample.  Defaults to \code{"basic"}.  See 
%     \code{\link[boot]{boot.ci}} for details.}
  \item{show}{a character vector containing the names of HWE test
    statistics to display from the set of "D", "D'", "r", and "table".}
  \item{...}{ optional parameters passed to \code{HWE.test} (data.frame
    method) or \code{chisq.test} (base method).}
  \item{do.Allele.Freq}{logicial indication whether to summarize allele
    frequencies.}
  \item{do.HWE.test}{logicial indication whether to perform HWE tests}
}
\details{
  HWE.test calls \code{\link{diseq}} to computes the Hardy-Weinberg
  (dis)equilibrium statistics D, D', and r (correlation coefficient).
  Next it calls \code{\link{diseq.ci}} to compute a bootstrap confidence
  interval for these estimates.  Finally, it calls
  \code{\link{chisq.test}} to compute a p-value for Hardy-Weinberg
  Equilibrium using a simulation/permutation method.

  Using bootstrapping for the confidence interval and simulation for the
  p-value avoids reliance on the assumptions the underlying Chi-square
  approximation.  This is particularly important when some allele pairs
  have small counts.

  For details on the definition of D, D', and r, see the help page for
  \code{\link{diseq}}.
}
\value{
  An object of class \code{HWE.test} with components
  \item{diseq}{A \code{\link{diseq}} object providing details on the
    disequilibrium estimates.}
  \item{ci}{A \code{\link{diseq.ci}} object providing details on the
    bootstrap confidence intervals for the disequilibrium estimates.}
  \item{test}{A \code{htest} object providing details on the
    permutation based Chi-square test.}
  \item{call}{function call used to creat this object.}
  \item{conf, B, ci.B, simulate.p.value}{values used for these arguments.}
}
\author{ Gregory R. Warnes \email{greg@warnes.net }
  }

\seealso{
  \code{\link{genotype}},
  \code{\link{diseq}},
  \code{\link{diseq.ci}},
  \code{\link{HWE.chisq}},
  \code{\link{HWE.exact}},
  \code{\link[stats]{chisq.test}}
}


\examples{
\testonly{
set.seed(4657613)
}
## Marker with two alleles:
example.data   <- c("D/D","D/I","D/D","I/I","D/D",
                    "D/D","D/D","D/D","I/I","")
g1  <- genotype(example.data)
g1

HWE.test(g1)

## Compare with individual calculations:
diseq(g1)
diseq.ci(g1)
HWE.chisq(g1)
HWE.exact(g1)


## Marker with three alleles: A, C, and T
three.data   <- c(rep("A/A",16),
                  rep("C/A",40),
                  rep("C/T",40),
                  rep("C/C",20),
                  rep("T/T",6))

g3  <- genotype(three.data)
g3

HWE.test(g3, ci.B=10000)
}
\keyword{ misc }
%%\keyword{genetics}

% $Id: homozygote.Rd 1346 2011-02-02 02:21:44Z warnes $

\name{homozygote}
\alias{homozygote}
\alias{heterozygote}
\alias{carrier}
\alias{carrier.genotype}
\alias{allele}
\alias{allele.count}
\alias{allele.names}
\title{Extract Features of Genotype objects}
\description{
  \code{homozygote} creates an vector of logicals that are true when the
  alleles of the corresponding observation are the identical.
  
  \code{heterozygote } creates an vector of logicals that are true when the
  alleles of the corresponding observation differ.
  
  \code{carrier} create a logical vector or matrix of logicals
  indicating whether the specified alleles are present.
  
  \code{allele.count} returns the number of copies of the specified
  alleles carried by each observation.
  
  \code{allele} extract the specified allele(s) as a character vector
  or a 2 column matrix.

  \code{allele.names} extract the set of allele names.
 }
\usage{
homozygote(x,  allele.name, ...)
heterozygote(x, allele.name, ...)
carrier(x, allele.name, ...)
\method{carrier}{genotype}(x, allele.name=allele.names(x),
        any=!missing(allele.name), na.rm=FALSE, ...)
allele.count(x, allele.name=allele.names(x),any=!missing(allele.name),
             na.rm=FALSE)
allele(x, which=c(1,2) )
allele.names(x)
}
\arguments{
  \item{x}{ \code{genotype} object }
  \item{\dots}{ optional parameters (ignored) }
  \item{allele.name}{ character value or vector of allele names}
  \item{any}{ logical value.  When \code{TRUE}, a single count or
    indicator is returned by combining the results for all of the
    elements of \code{allele}. If \code{FALSE} separate counts or
    indicators should be returned for each element of
    \code{allele}.  Defaults to \code{FALSE} if \code{allele} is
    missing. Otherwise defaults to \code{TRUE}.}
  \item{na.rm}{ logical value indicating whether to remove missing
    values.  When true, any \code{NA} values will be replaced by
    \code{0} or \code{FALSE} as appropriate.  Defaults to \code{FALSE}.}
  \item{which}{ selects which allele to return. For first allele use
    \code{1}.  For second allele use \code{2}.  For both (the default)
    use \code{c(1,2)}.}
      
}

\details{
  When the \code{allele.name} argument is given, heterozygote and
  homozygote return \code{TRUE} if \emph{exactly} one or both alleles,
  respectively, match the specified allele.name.
}

\value{
  \code{homozygote} and \code{heterozygote } return a vector of
  logicals.

  \code{carrier} returns a logical vector if only one allele is
  specified, or if \code{any} is \code{TRUE}.  Otherwise, it returns
  matrix of logicals with one row for each element of \code{allele}.

  
  \code{allele.count} returns a  vector of counts if only one allele is
  specified, or if \code{any} is \code{TRUE}.  Otherwise, it returns
  matrix of counts with one row for each element of \code{allele}.
  
  \code{allele} returns a character vector when one allele is
  specified.  When 2 alleles are specified, it returns a 2 column
  character matrix.

  \code{allele.names} returns a character vector containing the set of
  allele names.
  
  
}
\author{ Gregory R. Warnes \email{greg@warnes.net} }

\seealso{
  \code{\link{genotype}},
  \code{\link{HWE.test}},
%  \code{\link{allele}},
%  \code{\link{homozygote}},
%  \code{\link{heterozygote}}, 
%  \code{\link{carrier}},
  \code{\link{summary.genotype}},
  %  \code{\link{allele.count}}
  \code{\link{locus}}
  \code{\link{gene}}
  \code{\link{marker}}
  }


  \examples{

example.data   <- c("D/D","D/I","D/D","I/I","D/D","D/D","D/D","D/D","I/I","")
g1  <- genotype(example.data)
g1

heterozygote(g1)
homozygote(g1)

carrier(g1,"D")
carrier(g1,"D",na.rm=TRUE)

# get count of one allele 
allele.count(g1,"D")

# get count of each allele
allele.count(g1)  # equivalent to
allele.count(g1, c("D","I"), any=FALSE)

# get combined count for both alleles
allele.count(g1,c("I","D"))

# get second allele
allele(g1,2)

# get both alleles
allele(g1)

}
\keyword{ misc }
%%\keyword{genetics}%-- one or more ...

% $Id: gregorius.Rd 1346 2011-02-02 02:21:44Z warnes $

\name{gregorius}
\alias{gregorius}
\title{Probability of Observing All Alleles with a Given Frequency in a
  Sample of a Specified Size.}
\description{
  Probability of observing all alleles with a given frequency in a
  sample of a specified size.
}
\usage{
gregorius(freq, N, missprob, tol = 1e-10, maxN = 10000, maxiter=100, showiter = FALSE)
}
\arguments{
  \item{freq}{(Minimum) Allele frequency (required)}
  \item{N}{Number of sampled genotypes}
  \item{missprob}{Desired maximum probability of failing to observe an allele.}
  \item{tol}{Omit computation for terms which contribute less than this value.}
  \item{maxN}{Largest value to consider when searching for N.}
  \item{maxiter}{Maximum number of iterations to use when searching for N.}
  \item{showiter}{Boolean flag indicating whether to show the iterations
    performed when searching for N.}
}
\details{
  If \code{freq} and \code{N} are provided, but \code{missprob} is omitted,
  this function computes the probability of failing to observe all alleles
  with true underlying frequency \code{freq} when \code{N} diploid
  genotypes are sampled.  This is accomplished using the sum provided in
  Corollary 2 of Gregorius (1980), omitting terms which contribute less
  than \code{tol} to the result.

  When \code{freq} and \code{missprob} are provide, but \code{N} is
  omitted. A binary search on the range of [1,\code{maxN}] is performed
  to locate the smallest sample size, \code{N}, for which the
  probability of failing to observe all alleles with true
  underlying frequency \code{freq} is at most \code{missprob}.  In this
  case, \code{maxiter} specifies the largest number of iterations to use
  in the binary search, and \code{showiter} controls whether the
  iterations of the search are displayed.
}
\value{
  A list containing the following values:
  \item{call}{   Function call used to generate this object.}
  \item{method}{ One of the strings, "Compute missprob given N and freq",
    or "Determine minimal N given missprob and freq", indicating which
    type of computation was performed.}
  \item{retval$freq}{ Specified allele frequency.}
  \item{retval$N}{    Specified or computed sample size. }
  \item{retval$missprob}{ Computed probability of failing to observe all
  of the alleles with frequency \code{freq}. }
}
\references{
  Gregorius, H.R. 1980. The probability of losing an allele when
  diploid genotypes are sampled.  Biometrics 36, 643-652.
}
\note{
  This code produces sample sizes that are slightly larger than those
  given in table 1 of Gregorius (1980).  This appears to be due to
  rounding of the computed \code{missprob}s by the authors of that
  paper.
  }
    
\author{ Code submitted by David Duffy \email{davidD@qumr.edu.au},
  substantially enhanced by Gregory R. Warnes
  \email{greg@warnes.net}. }
%\seealso{ ~~objects to SEE ALSO as \code{\link{~~fun~~}}, ~~~ }
\examples{

# Compute the probability of missing an allele with frequency 0.15 when
# 20 genotypes are sampled:
gregorius(freq=0.15, N=20)

# Determine what sample size is required to observe all alleles with true
# frequency 0.15 with probability 0.95
gregorius(freq=0.15, missprob=1-0.95)


}
\keyword{misc}

\name{expectedGenotypes}

\alias{expectedGenotypes}
\alias{expectedHaplotypes}

\title{Construct expected genotypes/haplotypes according to known allele
  variants}

\description{
  \code{expectedGenotypes} constructs expected genotypes according to
  known allele variants, which can be quite tedious with large number of
  allele variants. It can handle different level of ploidy.
}

\usage{
expectedGenotypes(x, alleles=allele.names(x), ploidy=2, sort=TRUE,
                  haplotype=FALSE)
expectedHaplotypes(x, alleles=allele.names(x), ploidy=2, sort=TRUE,
                   haplotype=TRUE)
}

\arguments{
  \item{x}{genotype or haplotype}
  \item{alleles}{character, vector of allele names}
  \item{ploidy}{numeric, number of chromosome sets i.e. 2 for human autosomal
    genes}
  \item{sort}{logical, sort genotypes according to order of alleles in
    \code{alleles} argument}
  \item{haplotype}{logical, construct haplotypes i.e. ordered genotype}
  At least one of \code{x} or \code{alleles} must be given.
}

\details{

\code{expectedHaplotypes()} just calls \code{expectedGenotypes()} with
argument \code{haplotype=TRUE}.

}

\value{

A character vector with genotype names as "alele1/alele2" for diploid
example. Length of output is \eqn{(n*(n+1))/2} for genotype (unordered
genotype) and \eqn{n*n} for haplotype (ordered genotype) for \eqn{n}
allele variants.

}

\author{ Gregor Gorjanc }

\seealso{ \code{\link{allele.names}}, \code{\link{genotype}}}

\examples{
  ## On genotype
  prp <- c("ARQ/ARQ", "ARQ/ARQ", "ARR/ARQ", "AHQ/ARQ", "ARQ/ARQ")
  alleles <- c("ARR", "AHQ", "ARH", "ARQ", "VRR", "VRQ")
  expectedGenotypes(as.genotype(prp))
  expectedGenotypes(as.genotype(prp, alleles=alleles))
  expectedGenotypes(as.genotype(prp, alleles=alleles, reorder="yes"))

  ## Only allele names
  expectedGenotypes(alleles=alleles)
  expectedGenotypes(alleles=alleles, ploidy=4)

  ## Haplotype
  expectedHaplotypes(alleles=alleles)
  expectedHaplotypes(alleles=alleles, ploidy=4)[1:20]
}

\keyword{ manip }
